/****************************************************************************************/
/* Led.ino                                                                              */
/* Collection of led routines                                                           */
/* SUB_POWER_OFF:           Subroutine to switch device in power off state              */
/* SUB_MAIN_COLOR:          Subroutine to adjust static color                           */
/* SUB_PROG_SEL:            Subroutine to adjust program                                */
/* SUB_RCL_STORAGE:         Subroutine to recall one of the five selected colors in FLAH*/
/* SUB_SET_STORAGE:         Subroutine to save one of the five selected colors in FLASH */
/* SUB_TEST_VALUE:          Subroutine to set test value                                */
/****************************************************************************************/


/****************************************************************************************/
/* Subroutine to switch device in power off state                                       */
/****************************************************************************************/
void SUB_POWER_OFF() 
{
  #ifdef _DEBUG_
    Serial.println ("");
    Serial.println ("Switching power off ..."); 
  # endif 
  SUB_DMX_SEND (0,0,0);                           /* Switch all colors off */
  if (UDP_COLOR == false)                         /* No unsaved color ? --> Save */
  {
    #ifdef _DEBUG_
      Serial.print ("Saving actual program: ");
      Serial.println (ACT_PROGRAM);
    # endif 
    EEPROM.write(0, ACT_PROGRAM);                 /* Save actual program in FLASH */
    EEPROM.commit();                              /* Store values permanently in FLASH */
    /* Single color selected ? */
    if ((ACT_PROGRAM >=1) && (ACT_PROGRAM <= 13)) /* Program for single color? */
    {
      #ifdef _DEBUG_
        Serial.print ("Saving actual colour dimming value: ");
        Serial.println (COLOR_DIM_STEP);
      # endif
      EEPROM.write(1, COLOR_DIM_STEP);            /* Save actual color dimming value in FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */
    }     
    /* Program selected ? */
    if ((ACT_PROGRAM >= 20) && (ACT_PROGRAM <= 29))           
    {
      #ifdef _DEBUG_
        Serial.print ("Saving actual program dimming value: ");
        Serial.println (PROG_DIM_STEP);
        Serial.print ("Saving actual program speed value: ");
        Serial.println (PROG_SPEED);      
      # endif
      EEPROM.write(2, PROG_DIM_STEP);             /* Save actual program dimming value in FLASH */
      EEPROM.write(3, PROG_SPEED);                /* Save program speed in FLASH */ 
      EEPROM.commit();                            /* Store values permanently in FLASH */
    }
  }
  else                                            /* Unsaved color ? --> No savings */
  {
    UDP_COLOR = false;                            /* Reset UDP_COLOR flag */
  }
  POWER_OFF = true;                               /* Set power off flag true */
  POWER_WEB_OFF = true;                           /* Set power off flag true for web */ 
  SELECT = 255;                                   /* Delete key buffer */
}


/****************************************************************************************/
/* Subroutine Main colors                                                               */
/****************************************************************************************/
void SUB_MAIN_COLOR()
{
  ACT_PROGRAM = SELECT;                           /* Save actual program */
  #ifdef _DEBUG_
    Serial.println ("");
  # endif
  if (SELECT==1)                                  /* SELECT = 1 = Cold white */
  {
    #ifdef _DEBUG_
      Serial.println ("Color cold white selected ...");
    # endif
    MAX_RED_COLOR = 252;                          /* Set maximum red color */          
    MAX_GREEN_COLOR = 252;                        /* Set maximum green color */ 
    MAX_BLUE_COLOR = 252;                         /* Set maximum blue color */
  }
  if (SELECT==2)                                  /* SELECT = 2 = Warm white */
  {
    #ifdef _DEBUG_
      Serial.println ("Color warm white selected ...");
    # endif
    MAX_RED_COLOR = 252;                          /* Set maximum red color */          
    MAX_GREEN_COLOR = 252;                        /* Set maximum green color */ 
    MAX_BLUE_COLOR = 126;                         /* Set maximum blue color */
  }
  if (SELECT==3)                                  /* SELECT = 3 = Yellow */
  {
    #ifdef _DEBUG_
      Serial.println ("Color yellow selected ...");
    # endif
    MAX_RED_COLOR = 252;                          /* Set maximum red color */          
    MAX_GREEN_COLOR = 252;                        /* Set maximum green color */ 
    MAX_BLUE_COLOR = 0;                           /* Set maximum blue color */
  }
  if (SELECT==4)                                  /* SELECT = 4 = Orange */
  {
    #ifdef _DEBUG_
      Serial.println ("Color orange selected ...");
    # endif
    MAX_RED_COLOR = 252;                          /* Set maximum red color */          
    MAX_GREEN_COLOR = 72;                         /* Set maximum green color */ 
    MAX_BLUE_COLOR = 0;                           /* Set maximum blue color */
  }
  if (SELECT==5)                                  /* SELECT = 5 = Red */
  {
    #ifdef _DEBUG_
      Serial.println ("Color red selected ...");
    # endif
    MAX_RED_COLOR = 252;                          /* Set maximum red color */          
    MAX_GREEN_COLOR = 0;                          /* Set maximum green color */ 
    MAX_BLUE_COLOR = 0;                           /* Set maximum blue color */
  }    
  if (SELECT==6)                                  /* SELECT = 6 = Dark purple */  
  {
    #ifdef _DEBUG_
      Serial.println ("Color dark purple selected ...");
    # endif
    MAX_RED_COLOR = 252;                          /* Set maximum red color */          
    MAX_GREEN_COLOR = 0;                          /* Set maximum green color */ 
    MAX_BLUE_COLOR = 72;                          /* Set maximum blue color */
  }  
  if (SELECT==7)                                  /* SELECT = 7 = purple */
  {
    #ifdef _DEBUG_
      Serial.println ("Color purple selected ...");
    # endif
    MAX_RED_COLOR = 252;                          /* Set maximum red color */          
    MAX_GREEN_COLOR = 0;                          /* Set maximum green color */ 
    MAX_BLUE_COLOR = 126;                         /* Set maximum blue color */
  }   
  if (SELECT==8)                                  /* SELECT = 8 = Bright purple */
  {
    #ifdef _DEBUG_
      Serial.println ("Color bright purple selected ...");
    # endif
    MAX_RED_COLOR = 252;                          /* Set maximum red color */          
    MAX_GREEN_COLOR = 0;                          /* Set maximum green color */ 
    MAX_BLUE_COLOR = 252;                         /* Set maximum blue color */
  }   
  if (SELECT==9)                                  /* SELECT = 9 = Blue */
  {
    #ifdef _DEBUG_
      Serial.println ("Color blue selected ...");
    # endif
    MAX_RED_COLOR = 0;                            /* Set maximum red color */          
    MAX_GREEN_COLOR = 0;                          /* Set maximum green color */ 
    MAX_BLUE_COLOR = 252;                         /* Set maximum blue color */ 
  }     
  if (SELECT==10)                                 /* SELECT = 10 = Bright blue */
  {
    #ifdef _DEBUG_
      Serial.println ("Color bright blue selected ...");
    # endif
    MAX_RED_COLOR = 0;                            /* Set maximum red color */          
    MAX_GREEN_COLOR = 54;                         /* Set maximum green color */ 
    MAX_BLUE_COLOR = 252;                         /* Set maximum blue color */ 
  }   
  if (SELECT==11)                                 /* SELECT = 11 = Cyan */
  {
    #ifdef _DEBUG_
      Serial.println ("Color cyan selected ...");
    # endif
    MAX_RED_COLOR = 0;                            /* Set maximum red color */          
    MAX_GREEN_COLOR = 252;                        /* Set maximum green color */ 
    MAX_BLUE_COLOR = 252;                         /* Set maximum blue color */ 
  }  
  if (SELECT==12)                                 /* SELECT = 12 = Bright green */
  {
    #ifdef _DEBUG_
      Serial.println ("Color bright green selected ...");
    # endif
    MAX_RED_COLOR = 90;                           /* Set maximum red color */          
    MAX_GREEN_COLOR = 252;                        /* Set maximum green color */ 
    MAX_BLUE_COLOR = 0;                           /* Set maximum blue color */ 
  }  
  if (SELECT==13)                                 /* SELECT = 13 = Green */
  {
    #ifdef _DEBUG_
      Serial.println ("Color green selected ...");
    # endif
    MAX_RED_COLOR = 0;                            /* Set maximum red color */          
    MAX_GREEN_COLOR = 252;                        /* Set maximum green color */ 
    MAX_BLUE_COLOR = 0;                           /* Set maximum blue color */ 
  }  
  
  RED_DIM = MAX_RED_COLOR/9;                      /* Set red dimmer value */
  GREEN_DIM = MAX_GREEN_COLOR/9;                  /* Set green dimmer value */
  BLUE_DIM = MAX_BLUE_COLOR/9;                    /* Set blue dimmer value */
  /* Calculate red color value */ 
  RED_COLOR = MAX_RED_COLOR - COLOR_DIM_STEP * RED_DIM;
  /* Calculate green color value */       
  GREEN_COLOR = MAX_GREEN_COLOR - COLOR_DIM_STEP * GREEN_DIM;
  /* Calculate blue color value */ 
  BLUE_COLOR = MAX_BLUE_COLOR - COLOR_DIM_STEP * BLUE_DIM;    
  /* Call subroutine send via DMX interface */         
  SUB_DMX_SEND (RED_COLOR, GREEN_COLOR, BLUE_COLOR);    
  delay (100);                                    /* Wait 100ms before new key can be read */
  SELECT = 255;                                   /* Delete key buffer */
}


/****************************************************************************************/
/* Subroutine Program SELECT                                                            */
/****************************************************************************************/
void SUB_PROG_SEL()
{
  boolean BREAK_LOOP = false;                     /* Break flag for leaving subroutine */  
  boolean PAUSE_LOOP = false;                     /* Pause flag when Pause has been pressed */
  byte RED_DIM_DIR;                               /* Dimming of red colour 0 = Keep constant, 1 = Dim up, 2 = Dim down */
  byte GREEN_DIM_DIR;                             /* Dimming of green colour 0 = Keep constant, 1 = Dim up, 2 = Dim down */
  byte BLUE_DIM_DIR;                              /* Dimming of blue colour 0 = Keep constant, 1 = Dim up, 2 = Dim down */ 
  byte MAX_STEP;                                  /* Number of steps in sequence */
  byte STEP_COUNT;                                /* Actual step in sequence */
  byte NEXT_RED_COLOR;                            /* Red color of next step */
  byte NEXT_GREEN_COLOR;                          /* Green color of next step */
  byte NEXT_BLUE_COLOR;                           /* Blue color of next step */
  byte RED_DIM_RANGE;                             /* From to value of dimming of red color */
  byte GREEN_DIM_RANGE;                           /* From to value of dimming of green color */
  byte BLUE_DIM_RANGE;                            /* From to value of dimming of blue color */
  unsigned int DIMMER_LOOP;                       /* Dimmer loop dimming up or down or hold time */
  unsigned int SEQUENZ_COUNT;                     /* Main pointer in array */
  unsigned int START_POS;                         /* Start pointer for choosen program in array */
  unsigned int HOLD_DELAY;                        /* Delay for hold between dimming up and down */
  unsigned int DIM_DELAY;                         /* Delay for dimming up or down */
  float EXP_CONST;                                /* Constant for exponential function for 200 steps */
  float RED_START_TIME;                           /* Starting time for exponential function for red color 0-200 */
  float GREEN_START_TIME;                         /* Starting time for exponential function for green color 0-200 */
  float BLUE_START_TIME;                          /* Starting time for exponential function for blue color 0-200 */
  float RED_STOP_TIME;                            /* Stopping time for exponential function for red color 0-200 */
  float GREEN_STOP_TIME;                          /* Stopping time for exponential function for green color 0-200 */
  float BLUE_STOP_TIME;                           /* Stopping time for exponential function for blue color 0-200 */
  float RED_STEP;                                 /* Step time for 200 steps dor red color */
  float GREEN_STEP;                               /* Step time for 200 steps dor green color */
  float BLUE_STEP;                                /* Step time for 200 steps dor blue color */
  float BRIGHT_VAL;                               /* Floating calculation value */

  #ifdef _DEBUG_
    Serial.println (""); 
  # endif
   
  do
  {
    ACT_PROGRAM = SELECT;                         /* Save sctual program value */
    SELECT = SELECT - 20;                         /* Determine correct program */
    #ifdef _DEBUG_
      Serial.print ("Program ");
      Serial.print (ACT_PROGRAM-20);
      Serial.print (" selected with dimming factor ");
      Serial.print (PROG_DIM_STEP);
      Serial.print (" and speed factor ");
      Serial.print (PROG_SPEED);
      Serial.println (" selected ...");
    # endif
        
    START_POS = PROG_POINTER[SELECT];             /* Determine start position in sequence array */
    /* Determine number of steps for choosen sequence */  
    MAX_STEP = pgm_read_byte_near(PROG_DATA + START_POS);      
    STEP_COUNT = 1;                               /* Initialise step counter */
    SEQUENZ_COUNT = START_POS +1;                 /* Initialise sequence counter */ 
    delay (250);                                  /* 250ms initial keyboard delay */
    SELECT = 255;                                 /* Delete key buffer */ 
    EXP_CONST = NO_DIM_STEPS/log(255);            /* Constant for exponential function */
    SUB_TRIGGER_WATCHDOG ();                      /* Call subroutine to trigger watchdog at central location in main loop */
    
    do                                            /* Repeat until break condition reached */
    {
      /* Read red color out of array */
      RED_COLOR = pgm_read_byte_near(PROG_DATA + SEQUENZ_COUNT);
      /* Multiply with dimming factor of array and divide by 100 */ 
      BRIGHT_VAL = RED_COLOR*DIM_CONST[PROG_DIM_STEP]/100;    
      RED_COLOR = BRIGHT_VAL;                     /* Conert to byte value */ 
      /* Read green color out of array */
      GREEN_COLOR = pgm_read_byte_near(PROG_DATA + SEQUENZ_COUNT+1);
      /* Multiply with dimming factor of array and divide by 100 */
      BRIGHT_VAL = GREEN_COLOR*DIM_CONST[PROG_DIM_STEP]/100;  
      GREEN_COLOR = BRIGHT_VAL;                   /* Conert to byte value */       
      /* Read blue color out of array */
      BLUE_COLOR = pgm_read_byte_near(PROG_DATA + SEQUENZ_COUNT+2);
      /* Multiply with dimming factor of array and divide by 100 */
      BRIGHT_VAL = BLUE_COLOR*DIM_CONST[PROG_DIM_STEP]/100;   
      BLUE_COLOR = BRIGHT_VAL;                    /* Conert to byte value */       
      /* Read dimming factor out of array */
      DIM_DELAY = pgm_read_byte_near(PROG_DATA + SEQUENZ_COUNT+3);
      BRIGHT_VAL = DIM_DELAY * 1000;              /* Convert to 1 ms */
      BRIGHT_VAL = BRIGHT_VAL / NO_DIM_STEPS;     /* Devide by number of dimming steps */
      /* Multiply with speed factor of array and divide by 100 */
      BRIGHT_VAL = BRIGHT_VAL * SPEED_CONST[PROG_SPEED]/100;  
      DIM_DELAY = BRIGHT_VAL;                     /* Conert to an integer */                                 
      /* Read delay factor out of array */
      HOLD_DELAY = pgm_read_byte_near(PROG_DATA + SEQUENZ_COUNT+4);
      BRIGHT_VAL = HOLD_DELAY * 100;              /* Convert to 10ms */
      /* Multiply with speed factor of array and divide by 100 */
      BRIGHT_VAL = BRIGHT_VAL * SPEED_CONST[PROG_SPEED]/100;  
      HOLD_DELAY = BRIGHT_VAL;                    /* Conert to an integer */
      
      /* Extract dimming factor and direction */
      if (STEP_COUNT < MAX_STEP)                  /* End of sequence not reached? */
      {
        /* Read red color of next step out of array */
        NEXT_RED_COLOR = pgm_read_byte_near(PROG_DATA + SEQUENZ_COUNT+5);
        /* Read green color of next step out of array */
        NEXT_GREEN_COLOR = pgm_read_byte_near(PROG_DATA + SEQUENZ_COUNT+6);
        /* Read blue color of next step out of array */
        NEXT_BLUE_COLOR = pgm_read_byte_near(PROG_DATA + SEQUENZ_COUNT+7);
      }
      else                                        /* End of sequence reached? */ 
      {
        /* Read red color of next step out of array */
        NEXT_RED_COLOR = pgm_read_byte_near(PROG_DATA + START_POS +1);
        /* Read green color of next step out of array */
        NEXT_GREEN_COLOR = pgm_read_byte_near(PROG_DATA + START_POS +2);
        /* Read blue color of next step out of array */
        NEXT_BLUE_COLOR = pgm_read_byte_near(PROG_DATA + START_POS +3); 
      }
      /* Multiply next red color with dimming factor of array and divide by 100 */
      BRIGHT_VAL = NEXT_RED_COLOR*DIM_CONST[PROG_DIM_STEP]/100;    
      NEXT_RED_COLOR = BRIGHT_VAL;                /* Conert to byte value */     
      /* Multiply next green color with dimming factor of array and divide by 100 */
      BRIGHT_VAL = NEXT_GREEN_COLOR*DIM_CONST[PROG_DIM_STEP]/100;    
      NEXT_GREEN_COLOR = BRIGHT_VAL;              /* Conert to byte value */  
      /* Multiply next blue color with dimming factor of array and divide by 100 */
      BRIGHT_VAL = NEXT_BLUE_COLOR*DIM_CONST[PROG_DIM_STEP]/100;    
      NEXT_BLUE_COLOR = BRIGHT_VAL;               /* Conert to byte value */ 

      if (RED_COLOR < NEXT_RED_COLOR)             /* Next value higher then previous one? */
      {
        RED_DIM_DIR = 1;                          /* Set dimming direction to 1 = Up */
      }
      else                                        /* Next value less then previous one */
      { 
        RED_DIM_DIR = 2;                          /* Set dimming direction to 2 = Down */      
      }
      if (RED_COLOR == NEXT_RED_COLOR)            /* No change in dimming in actual step ? */
      {
        RED_DIM_DIR = 0;                          /* Set dimming direction to 0 = Hold value */ 
        RED_START_TIME = 0;                       /* Set red starting time to 0 */
        RED_STOP_TIME = 200;                      /* Set red stopping time to 200 */
        RED_STEP = 1;                             /* Set red step to 1 */
      }
      else                                        /* Changes in actual step ? */
      {
        if (RED_COLOR == 0)                       /* Red color value is zero ? */
        {
          RED_START_TIME = 0;                     /* Set red starting time to 0 */                   
        }
        else                                      /* Red color value is not zero ? */
        {
          /* Calculate red starting time */
          RED_START_TIME = EXP_CONST*log(RED_COLOR);          
        } 
        if (NEXT_RED_COLOR == 0)                  /* Next red color value is zero ? */
        {
          RED_STOP_TIME = 0;                      /* Set red stopping time to 0 */
        }
        else                                      /* Next red color value is not zero ? */
        {
          /* Calculate next red starting time */
          RED_STOP_TIME = EXP_CONST*log(NEXT_RED_COLOR);      
        } 
        if (RED_DIM_DIR == 1)                     /* Dimming up chosen */
        {
          /* Calculating step value */
          RED_STEP = (RED_STOP_TIME - RED_START_TIME) / NO_DIM_STEPS;      
        }
        else                                      /* Dimming down chosen */
        {
          /* Calculating step value */          
          RED_STEP = (RED_START_TIME - RED_STOP_TIME) / NO_DIM_STEPS;      
        }        
      }
      if (GREEN_COLOR < NEXT_GREEN_COLOR)         /* Next value higher then previous one? */
      {
        GREEN_DIM_DIR = 1;                        /* Set dimming direction to 1 = Up */
      }
      else                                        /* Next value less then previous one */
      {
        GREEN_DIM_DIR = 2;                        /* Set dimming direction to 2 = Down */      
      }
      if (GREEN_COLOR == NEXT_GREEN_COLOR)        /* No change in dimming in actual step ? */
      {
        GREEN_DIM_DIR = 0;                        /* Set dimming direction to 0 = Hold value */ 
        GREEN_START_TIME = 0;                     /* Set green starting time to 0 */
        GREEN_STOP_TIME = 200;                    /* Set green stopping time to 200 */
        GREEN_STEP = 1;                           /* Set green step to 1 */
      }
      else                                        /* Changes in actual step ? */
      {
        if (GREEN_COLOR == 0)                     /* Green color value is zero ? */
        {
          GREEN_START_TIME = 0;                   /* Set green starting time to 0 */
        }
        else                                      /* Green color value is not zero ? */
        {
          /* Calculate green starting time */
          GREEN_START_TIME = EXP_CONST*log(GREEN_COLOR);      
        } 
        if (NEXT_GREEN_COLOR == 0)                /* Next green color value is zero ? */
        {
          GREEN_STOP_TIME = 0;                    /* Set green stopping time to 0 */
        }
        else                                      /* Next green color value is not zero ? */
        {
          /* Calculate next green starting time */
          GREEN_STOP_TIME = EXP_CONST*log(NEXT_GREEN_COLOR);  
        } 
        if (GREEN_DIM_DIR == 1)                   /* Dimming up chosen */
        {
          /* Calculating step value */          
          GREEN_STEP = (GREEN_STOP_TIME - GREEN_START_TIME) / NO_DIM_STEPS;      
        }
        else                                      /* Dimming down chosen */
        {
          /* Calculating step value */          
          GREEN_STEP = (GREEN_START_TIME - GREEN_STOP_TIME) / NO_DIM_STEPS;      
        }                 
      }
      if (BLUE_COLOR < NEXT_BLUE_COLOR)           /* Next value higher then previous one? */
      {
        BLUE_DIM_DIR = 1;                         /* Set dimming direction to 1 = Up */
      }
      else                                        /* Next value less then previous one */
      {
        BLUE_DIM_DIR = 2;                         /* Set dimming direction to 2 = Down */      
      }
      if (BLUE_COLOR == NEXT_BLUE_COLOR)          /* No change in dimming in actual step ? */
      {
        BLUE_DIM_DIR = 0;                         /* Set dimming direction to 0 = Hold value */ 
        BLUE_START_TIME = 0;                      /* Set blue starting time to 0 */
        BLUE_STOP_TIME = 200;                     /* Set blue stopping time to 200 */
        BLUE_STEP = 1;                            /* Set blue step to 1 */
      }
      else                                        /* Changes in actual step ? */
      {
        if (BLUE_COLOR == 0)                      /* Blue color value is zero ? */
        {
          BLUE_START_TIME = 0;                    /* Set blue starting time to 0 */
        }
        else                                      /* Blue color value is not zero ? */
        {
          /* Calculate blue starting time */
          BLUE_START_TIME = EXP_CONST*log(BLUE_COLOR);        
        } 
        if (NEXT_BLUE_COLOR == 0)                 /* Next blue color value is zero ? */
        {
          BLUE_STOP_TIME = 0;                     /* Set blue stopping time to 0 */
        }
        else                                      /* Next blue color value is not zero ? */
        {
          /* Calculate next blue starting time */
          BLUE_STOP_TIME = EXP_CONST*log(NEXT_BLUE_COLOR);    
        }   
        if (BLUE_DIM_DIR == 1)                    /* Dimming up chosen */
        {
          /* Calculating step value */
          BLUE_STEP = (BLUE_STOP_TIME - BLUE_START_TIME) / NO_DIM_STEPS;      
        }
        else                                      /* Dimming down chosen */
        {
          /* Calculating step value */
          BLUE_STEP = (BLUE_START_TIME - BLUE_STOP_TIME) / NO_DIM_STEPS;      
        }      
      }
                   
      /* Main loop for dimming up or down */
      DIMMER_LOOP = 0;                            /* Set dimmer loop to zero */
      do                                          /* Repeat unit dimming of actual step finished or break condition reached */
      {
        /* Check if power off by time */
        getLocalTime(&TIMEINFO);                  /* Get actual time */
        ACT_MIN         = TIMEINFO.tm_min;        /* Extract actual minute */ 
        ACT_HOUR        = TIMEINFO.tm_hour;       /* Extract actual hour */ 
       /* Check for switch off */
        if ((ACT_HOUR == 22) && (ACT_MIN == 00))
        {
          SELECT=128;                             /* Switch off */
        }  
        if (SELECT == 128)                        /* Power Off pressed */
        {
          BREAK_LOOP = true;                      /* Set break flag true */
        }
        if ((SELECT >= 48) && (SELECT <= 51))     /* Dimming key pressed */
        {
          BREAK_LOOP = true;                      /* Set break flag true */ 
        }
        if ((SELECT >= 64) && (SELECT <= 73))     /* Speed key pressed */
        {
          BREAK_LOOP = true;                      /* Set break flag true */
        }                
        if ((SELECT >= 1) && (SELECT <= 29))      /* One of the color, store or program keys pressed? */
        {
          BREAK_LOOP = true;                      /* Set break flag true */
        }  
        if (BOOTLOADER_FLAG == true)              /* Bootloader selected ? */
        {
          BREAK_LOOP = true;                      /* Set break flag true */
        }  
        if (RESET_FLAG == true)                   /* Reset selected ? */
        {
          BREAK_LOOP = true;                      /* Set break flag true */
        }                       
        if (SELECT == 130)                        /* Pause pressed */
        {
          if (PAUSE_LOOP == false)
          {
            #ifdef _DEBUG_
              Serial.println ("");
              Serial.println ("Pause selected ..."); 
            # endif
          }
          PAUSE_LOOP = true;                      /* Set pause loop true */
        }
        if (SELECT == 131)                        /* Play pressed */
        {
          if (PAUSE_LOOP == true)
          { 
            #ifdef _DEBUG_  
              Serial.println ("");
              Serial.println ("Play selected ..."); 
            # endif       
          }
          PAUSE_LOOP = false;                     /* Set pause loop true */
        }
        /* Main setting of colours */
        if (PAUSE_LOOP == false)                  /* Pause not pressed? */
        {
          /* Calculation of red colour */
          if (RED_DIM_DIR == 1)                   /* Dimming up? */
          {
            /* Get red value from exponential function */
            BRIGHT_VAL = exp(RED_START_TIME/EXP_CONST);       
            RED_START_TIME += RED_STEP;           /* Calculate next step */
            
          }
          if (RED_DIM_DIR == 2)                   /* Dimming down? */
          {
            /* Get red value from exponential function */
            BRIGHT_VAL = exp(RED_START_TIME/EXP_CONST);       
            RED_START_TIME -= RED_STEP;           /* Calculate next step */
          }
          if (RED_DIM_DIR > 0)                    /* Dimming up or down ? */
          {
            RED_COLOR = BRIGHT_VAL;               /* Convert value to a byte value */  
            if ((BRIGHT_VAL - RED_COLOR) > 0.5)   /* Rounded to byte value not correct ? */
            {
              RED_COLOR +=1;                      /* Correct red color value */
            }
          }
          /* Calculation of green colour */
          if (GREEN_DIM_DIR == 1)                 /* Dimming up? */
          {
            /* Get green value from exponential function */
            BRIGHT_VAL = exp(GREEN_START_TIME/EXP_CONST);     
            GREEN_START_TIME += GREEN_STEP;       /* Calculate next step */
          }
          if (GREEN_DIM_DIR == 2)                 /* Dimming down? */
          {
            /* Get green value from exponential function */
            BRIGHT_VAL = exp(GREEN_START_TIME/EXP_CONST);     
            GREEN_START_TIME -= GREEN_STEP;       /* Calculate next step */
          }
          if (GREEN_DIM_DIR > 0)                  /* Dimming up or down ? */
          {
            GREEN_COLOR = BRIGHT_VAL;             /* Convert value to a byte value */  
            if ((BRIGHT_VAL - GREEN_COLOR) > 0.5) /* Rounded to byte value not correct ? */
            {
              GREEN_COLOR +=1;                    /* Correct red color value */
            }
          }
          /* Calculation of blue colour */
          if (BLUE_DIM_DIR == 1)                  /* Dimming up? */
          {
            /* Get blue value from exponential function */
            BRIGHT_VAL = exp(BLUE_START_TIME/EXP_CONST);      
            BLUE_START_TIME += BLUE_STEP;         /* Calculate next step */
          }
          if (BLUE_DIM_DIR == 2)                  /* Dimming down? */
          {
            /* Get blue value from exponential function */
            BRIGHT_VAL = exp(BLUE_START_TIME/EXP_CONST);      
            BLUE_START_TIME -= BLUE_STEP;         /* Calculate next step */
          }
          if (BLUE_DIM_DIR > 0)                   /* Dimming up or down ? */
          {
            BLUE_COLOR = BRIGHT_VAL;              /* Convert value to a byte value */  
            if ((BRIGHT_VAL - BLUE_COLOR) > 0.5)  /* Rounded to byte value not correct ? */
            {
              BLUE_COLOR +=1;                     /* Correct red color value */
            }
          }
          /* Call subroutine send via DMX interface */
          SUB_DMX_SEND (RED_COLOR, GREEN_COLOR, BLUE_COLOR);  
          SUB_TRIGGER_WATCHDOG ();                /* Call subroutine to trigger watchdog at central location in main loop */
          delay (DIM_DELAY);                      /* Dimmer delay for each step */
          SUB_TRIGGER_WATCHDOG ();                /* Call subroutine to trigger watchdog at central location in main loop */

          DIMMER_LOOP = DIMMER_LOOP + 1;          /* Increase dimmer loop by one */
        }
        else                                      /* Pause pressed? */
        {
          SUB_TRIGGER_WATCHDOG ();                /* Call subroutine to trigger watchdog at central location in main loop */  
        }
      }
      while ((BREAK_LOOP == false) && (DIMMER_LOOP <= NO_DIM_STEPS));

      SUB_TRIGGER_WATCHDOG ();                    /* Call subroutine to trigger watchdog at central location in main loop */

      /* Switch off when zero before hold timer*/
      if (NEXT_RED_COLOR == 0)                    /* Next color for red color zero ? */
      {
        RED_COLOR = 0;                            /* Set red color to zero */
      }
      if (NEXT_GREEN_COLOR == 0)                  /* Next color for green color zero ? */
      {
        GREEN_COLOR = 0;                          /* Set green color to zero */
      }
      if (NEXT_BLUE_COLOR == 0)                   /* Next color for blue color zero ? */
      {
        BLUE_COLOR = 0;                           /* Set blue color to zero */
      }
      /* Call subroutine send via DMX interface */ 
      SUB_DMX_SEND (RED_COLOR, GREEN_COLOR, BLUE_COLOR);      

      /* Main loop for hold delay */
      DIMMER_LOOP = 0;                            /* Set dimmer loop to zero */
      do                                          /* Repeat unit dimming of actual step finished or break condition reached */
      {
        /* Check if power off by time */
        getLocalTime(&TIMEINFO);                  /* Get actual time */
        ACT_MIN         = TIMEINFO.tm_min;        /* Extract actual minute */ 
        ACT_HOUR        = TIMEINFO.tm_hour;       /* Extract actual hour */ 
        
       /* Check for switch off */
        if ((ACT_HOUR == 22) && (ACT_MIN == 00))
        {
          SELECT=128;                             /* Switch off */
        }  
        if (SELECT == 128)                        /* Power Off pressed */
        {
          BREAK_LOOP = true;                      /* Set break flag true */
        }
        if ((SELECT >= 48) && (SELECT <= 51))     /* Dimming key pressed */
        {
          BREAK_LOOP = true;                      /* Set break flag true */ 
        }
        if ((SELECT >= 64) && (SELECT <= 73))     /* Speed key pressed */
        {
          BREAK_LOOP = true;                      /* Set break flag true */
        }                
        if ((SELECT >= 1) && (SELECT <= 29))      /* One of the color, store or program keys pressed? */
        {
          BREAK_LOOP = true;                      /* Set break flag true */
        }   
        if (BOOTLOADER_FLAG == true)              /* Bootloader selected ? */
        {
          BREAK_LOOP = true;                      /* Set break flag true */
        }     
        if (RESET_FLAG == true)                   /* Reset selected ? */
        {
          BREAK_LOOP = true;                      /* Set break flag true */
        }                     
        if (SELECT == 130)                        /* Pause pressed */
        {
          if (PAUSE_LOOP == false)
          {
            #ifdef _DEBUG_
              Serial.println ("");
              Serial.println ("Pause selected ..."); 
            # endif
          }
          PAUSE_LOOP = true;                      /* Set pause loop true */
        }
        if (SELECT == 131)                        /* Play pressed */
        {
          if (PAUSE_LOOP == true)
          {  
            #ifdef _DEBUG_
              Serial.println ("");
              Serial.println ("Play selected ..."); 
            # endif             
          }
          PAUSE_LOOP = false;                     /* Set pause loop true */
        }

        if (PAUSE_LOOP == false)                  /* Pause not pressed? */
        {      
          delay (10);                             /* 10ms delay */
          DIMMER_LOOP = DIMMER_LOOP + 1;          /* Increase dimmer loop by one */
        } 
        else                                      /* Pause pressed? */
        {
          SUB_TRIGGER_WATCHDOG ();                /* Call subroutine to trigger watchdog at central location in main loop */  
        }            
      }                            
      /* Repeat unit delay time reached or break condition reached */
      while ((BREAK_LOOP == false) && (DIMMER_LOOP <= HOLD_DELAY));
       
     /* Prepare next step */
     SEQUENZ_COUNT = SEQUENZ_COUNT + 5;           /* Increase sequenz counter */
     STEP_COUNT = STEP_COUNT + 1;                 /* Increase step counter */
     if (STEP_COUNT > MAX_STEP)                   /* Maximum steps reached? */
     {
       STEP_COUNT = 1;                            /* Reset step counter */
       SEQUENZ_COUNT = START_POS +1;              /* Reset sequence counter */
     }  
    }
    while (BREAK_LOOP == false);                  /* Repeat until break loop = true */    
      
    if ((SELECT >= 20) && (SELECT <= 29))         /* New program pressed */
    {
      BREAK_LOOP = false;                         /* Set break flag false */   
    }   
      
    if ((SELECT >= 48) && (SELECT <= 51))         /* Dimming key pressed */ 
    {
      PROG_DIM_STEP = SELECT-48;                  /* Calculate dimming value */
      #ifdef _DEBUG_
        Serial.println ("");
        Serial.print ("Program dimming ");
        Serial.print (SELECT-48);
        Serial.println (" selected ...");  
      # endif      
      SELECT = ACT_PROGRAM;                       /* Restart program */
      BREAK_LOOP = false;                         /* Set break flag false */   
    }   
    
    if ((SELECT >= 64) && (SELECT <= 73))         /* Speed key pressed */
    {
      PROG_SPEED = SELECT - 64;                   /* Calculate dimming value */
      #ifdef _DEBUG_      
        Serial.println ("");
        Serial.print ("Program speed ");
        Serial.print (SELECT-64);
        Serial.println (" selected ..."); 
      # endif  
      SELECT = ACT_PROGRAM;                       /* Restart program */
      BREAK_LOOP = false;                         /* Set break flag false */
    }     
  }
  while (BREAK_LOOP == false);                    /* Repeat until break loop = true */

  SUB_TRIGGER_WATCHDOG ();                        /* Call subroutine to trigger watchdog at central location in main loop */
      
  if (SELECT == 128)                              /* Power Off pressed */
  {
    SUB_POWER_OFF();                              /* Call subroutine for power off */
  } 
  else                                            /* Power Off not pressed */   
  {
    if ((SELECT >=1) && (SELECT <= 13))           /* Program for single color? */
    {
      ACT_PROGRAM = SELECT;                       /* Copy SELECT to sctual program */
      SUB_MAIN_COLOR();                           /* Call subroutine for single color */
    }
    
    /* Program for stored values? */
    if ((ACT_PROGRAM >= 15) && (ACT_PROGRAM <= 19))
    {
      ACT_PROGRAM = SELECT;                       /* Copy SELECT to sctual program */
      SUB_RCL_STORAGE();                          /* Call subroutine for single storage */   
    }
  }  
}


/****************************************************************************************/
/* Subroutine to recall one of the five selected colors in FLASH                        */
/****************************************************************************************/
void SUB_RCL_STORAGE()
{
  ACT_PROGRAM = SELECT;                           /* Copy SELECT variable to sctual program */
  SELECT = (ACT_PROGRAM - 14) * 10;               /* Calculate storage position */
  #ifdef _DEBUG_
    Serial.println ("");
    Serial.print ("Color storage position ");
    Serial.print (ACT_PROGRAM - 14);
    Serial.println (" recalled ...");
  # endif 
  MAX_RED_COLOR = EEPROM.read (SELECT);           /* Read maximum red color out of FLASH */
  SELECT = SELECT + 1;                            /* Calculate new storage position */
  MAX_GREEN_COLOR = EEPROM.read (SELECT);         /* Read maximumgreen color out of EEROM */
  SELECT = SELECT + 1;                            /* Calculate new storage position */
  MAX_BLUE_COLOR = EEPROM.read (SELECT);          /* Read maximum blue color out of EEPROM */                    
  SELECT = SELECT + 1;                            /* Calculate new storage position */
  COLOR_DIM_STEP =  EEPROM.read (SELECT);         /* Read dimmer value */
  
  RED_DIM = MAX_RED_COLOR/9;                      /* Set red dimmer value */
  GREEN_DIM = MAX_GREEN_COLOR/9;                  /* Set green dimmer value */
  BLUE_DIM = MAX_BLUE_COLOR/9;                    /* Set blue dimmer value */
  /* Calculate red color value */ 
  RED_COLOR = MAX_RED_COLOR - COLOR_DIM_STEP * RED_DIM;  
  /* Calculate green color value */     
  GREEN_COLOR = MAX_GREEN_COLOR - COLOR_DIM_STEP * GREEN_DIM; 
  /* Calculate blue color value */
  BLUE_COLOR = MAX_BLUE_COLOR - COLOR_DIM_STEP * BLUE_DIM;    
  /* Call subroutine send via DMX interface */          
  SUB_DMX_SEND (RED_COLOR, GREEN_COLOR, BLUE_COLOR);            
  delay (100);                                    /* Wait 100ms before new key can be read */
  SELECT = 255;                                   /* Reset Select value */
}  


/****************************************************************************************/
/* Subroutine to save one of the five selected colors in FLASH                          */
/****************************************************************************************/
void SUB_SET_STORAGE()
{
  ACT_PROGRAM = SELECT;                           /* Copy SELECT variable to sctual program */
  #ifdef _DEBUG_
    Serial.println ("");
    Serial.print ("Color storage position ");
    Serial.print (ACT_PROGRAM - 135);
    Serial.println (" stored...");
  # endif 
  SELECT = (ACT_PROGRAM - 135) * 10;              /* Calculate storage position */
  EEPROM.write(SELECT, MAX_RED_COLOR);            /* Write RED_COLOR */
  SELECT = SELECT + 1;                            /* Calculate next storage position */
  EEPROM.write(SELECT, MAX_GREEN_COLOR);          /* Write GREEN_COLOR */
  SELECT = SELECT + 1;                            /* Calculate next storage position */
  EEPROM.write(SELECT, MAX_BLUE_COLOR);           /* Write BLUE_COLOR */    
  SELECT = SELECT + 1;                            /* Calculate next storage position */  
  EEPROM.write(SELECT, COLOR_DIM_STEP);           /* Write dimmer value */
  EEPROM.commit();                                /* Store values permanently in EEPROM */ 
           
  ACT_PROGRAM = ACT_PROGRAM - 121;                /* Calculate correct actual program */
  UDP_COLOR = false;                              /* Reset UDP_COLOR flag */
  delay (100);                                    /* Wait 100ms before new key can be read */
  SELECT = 255;                                   /* Reset Select value */
} 


/****************************************************************************************/
/* Subroutine to set test value                                                         */
/****************************************************************************************/
void SUB_TEST_VALUE()
{  
  RED_DIM   = STORE_RED_COLOR/9;                  /* Set red dimmer value */
  GREEN_DIM = STORE_GREEN_COLOR/9;                /* Set green dimmer value */
  BLUE_DIM  = STORE_BLUE_COLOR/9;                 /* Set blue dimmer value */
  /* Calculate red color value */ 
  RED_COLOR = STORE_RED_COLOR - STORE_DIM_VALUE * RED_DIM;  
  /* Calculate green color value */     
  GREEN_COLOR = STORE_GREEN_COLOR - STORE_DIM_VALUE * GREEN_DIM; 
  /* Calculate blue color value */
  BLUE_COLOR = STORE_BLUE_COLOR - STORE_DIM_VALUE * BLUE_DIM;    
  /* Call subroutine send via DMX interface */           
  SUB_DMX_SEND (RED_COLOR, GREEN_COLOR, BLUE_COLOR);            
}  
