/****************************************************************************************/
/* Web.ino                                                                              */
/* Collection of subroutines to display and process webpages                            */
/* SUB_PROCESS_UDP_REQUEST: Subroutine to check UDP client request                      */
/* SUB_DECODE_MESSAGE:      Subroutine to decode client request other then webpages     */
/* SUB_DEFINE_ROUTES:       Subroutine to define the routes for ESPAsyncWebserver       */
/*                          to be once during setup                                     */
/* SUB_MESSAGE_ERROR:       Subroutine to be called if message error is detected        */   
/* SUB_WEB_PROCESSOR_MAIN:  Subroutine to replace placeholder in Websites               */
/*                          with calculated values                                      */  
/* SUB_WEB_PROCESSOR_SUBM1: Subroutine to replace placeholder in website submenu 1      */
/*                          with calculated values                                      */
/* SUB_WEB_PROCESSOR_SUBM2: Subroutine to replace placeholder in website submenu 2      */
/*                          with calculated values                                      */       
/* SUB_WEB_PROCESSOR_SUBM3: Subroutine to replace placeholder in website submenu 3      */
/*                          with calculated values                                      */   
/* SUB_WEB_PROCESSOR_SUBM4: Subroutine to replace placeholder in website submenu 4      */
/*                          with calculated values                                      */
/* SUB_WEB_PROCESSOR_SUBM7:Subroutine to replace placeholder in website submenu 4A     */
/*                          with calculated values (Recall storage)                     */   
/* SUB_WEB_PROCESSOR_SUBM5: Subroutine to replace placeholder in website submenu 5      */
/*                          with calculated values                                      */     
/****************************************************************************************/


/*************************************************************************************************************************************************/
/* Subroutine to check UDP client request                                                                                                        */
/*************************************************************************************************************************************************/
void SUB_PROCESS_UDP_REQUEST (void)
{
  boolean ERROR_FLAG;                             /* Error Flag,False = No error, True = Error */
  byte MODULO_CHECK;                              /* Modulo checker */
  char INPUT_CHAR;                                /* Read char from Ethernet */
  
  #ifdef _DEBUG_
    Serial.println("");               
    Serial.println("New UDPP client request ...");
  # endif  
  /* Step 1: Check on right length */
  if (UDP_PACKETS != 22)                          /* Correct number of bytes received ? */
  {
    #ifdef _DEBUG_
      Serial.println ("ERROR: Wrong number of bytes received!!!");
    # endif
    udpclient.flush ();                           /* Clear UDP buffer */   
  }
  else                                            /* Right number of bytes received ? */
  {
    RECEIVE_BUFFER = "";                          /* Clear receive buffer */
    while (udpclient.available () > 0)
    {
      INPUT_CHAR = udpclient.read ();             /* Read buffer as char */
      RECEIVE_BUFFER += INPUT_CHAR;               /* Save data in buffer string */     
    }
    #ifdef _DEBUG_
      Serial.print ("Received message: *");
      Serial.print (RECEIVE_BUFFER); 
      Serial.print ("*, "); 
      Serial.print ("String length:");
      Serial.println (UDP_PACKETS); 
    # endif
    ERROR_FLAG = false;                           /* Ser error flag false */
    /* Step 2: Look for Art-Net command */
    TEST_POS_1 = RECEIVE_BUFFER.indexOf ('Art-Net', 0); 
    if (TEST_POS_1 == 0)                          /* Art-Net not detected ? */
    {
      ERROR_FLAG = true;                          /* Set error flag true = Error */
      #ifdef _DEBUG_
        Serial.println ("ERROR: Art-Net command not found!!!");
      # endif       
    }
    /* Step 3 Check Art DMX datapacket */
    if (ERROR_FLAG ==false)                       /* No error ? */
    {          
      INPUT_CHAR = RECEIVE_BUFFER [TEST_POS_1+7];
      if (INPUT_CHAR != 80)                       /* 50H value detected? */
      {
        ERROR_FLAG = true;                        /* Set error flag true */    
      }
      /* Extract lower byte */ 
      INPUT_CHAR = RECEIVE_BUFFER [TEST_POS_1+8];          
      if (INPUT_CHAR != 0)                        /* Zero value detected? */
      {
        ERROR_FLAG  = true;                       /* Set error flag true */
      }
      if (ERROR_FLAG == true)
      {
        #ifdef _DEBUG_
          Serial.println ("ERROR: Wrong DMX datapacket!!!");
        # endif
      }
    }  
    /* Step 4 Check Artnet Version Number */
    if (ERROR_FLAG == false)                      /* No error yet ? */
    {
      /* Extract higher byte */ 
      INPUT_CHAR = RECEIVE_BUFFER [TEST_POS_1+9];          
      if (INPUT_CHAR != 14)                       /* 0EH value detected? */
      {
        ERROR_FLAG = true;                        /* Set error flag true */
      }
      /* Extract lower byte */ 
      INPUT_CHAR = RECEIVE_BUFFER [TEST_POS_1+10];         
      if (INPUT_CHAR != 0)                        /* Zero value detected? */
      {
        ERROR_FLAG = true;                        /* Set error flag true */
      }
      if (ERROR_FLAG == true)
      {
        #ifdef _DEBUG_
          Serial.println ("ERROR: Wrong Artnet version number received!!!");
        # endif
      }        
    } 
    /* Step 5 Check Message length */
    if (ERROR_FLAG == false)                      /* No error yet ? */
    {
      /* Extract lower byte */
      INPUT_CHAR = RECEIVE_BUFFER [TEST_POS_1+15];         
      /* Value not three or four? */
      if ((INPUT_CHAR != 3) && (INPUT_CHAR != 4))           
      {
        ERROR_FLAG = true;                        /* Set error flag true */
        #ifdef _DEBUG_
          Serial.println ("ERROR: Wrong Message length received!!!");
        # endif
      }
    }   
    /* Step 6 Extract DMX channel values */
    if (ERROR_FLAG == false)                      /* No error yet ? */
    {
      /* Extract first DMX channel */
      INPUT_CHAR = RECEIVE_BUFFER [TEST_POS_1+16];          
      RED_COLOR = INPUT_CHAR;                     /* Convert char to byte */
      #ifdef _DEBUG_
        Serial.print ("Valid colours received: Red colour: ");
        Serial.print (RED_COLOR);
      # endif
      /* Extract second DMX channel */
      INPUT_CHAR = RECEIVE_BUFFER [TEST_POS_1+17];           
      GREEN_COLOR = INPUT_CHAR;                   /* Convert char to byte */
      #ifdef _DEBUG_
        Serial.print (" Green colour: ");
        Serial.print (GREEN_COLOR);
      # endif
      /* Extract third DMX channel */   
      INPUT_CHAR = RECEIVE_BUFFER [TEST_POS_1+18];          
      BLUE_COLOR = INPUT_CHAR;                    /* Convert char to byte */ 
      #ifdef _DEBUG_
        Serial.print (" Blue colour: ");
        Serial.println (BLUE_COLOR);
      # endif
      COLOR_DIM_STEP = 0;                         /* Reset dimming factor */     
      MODULO_CHECK = RED_COLOR % 9;               /* Detremine modulo of 9 */
      if (MODULO_CHECK < 5)
      {
        RED_COLOR = RED_COLOR - MODULO_CHECK;     /* Round to next lower number */         
      }
      else
      {
        RED_COLOR = RED_COLOR + (9-MODULO_CHECK); /* Round to next higher number */       
      }
      MODULO_CHECK = GREEN_COLOR % 9;             /* Detremine modulo of 9 */
      if (MODULO_CHECK < 5)
      {
        GREEN_COLOR = GREEN_COLOR - MODULO_CHECK; /* Round to next lower number */     
      }
      else
      {
        /* Round to next higher number */   
        GREEN_COLOR = GREEN_COLOR + (9-MODULO_CHECK);     
      }   
      MODULO_CHECK = BLUE_COLOR % 9;              /* Detremine modulo of 9 */
      if (MODULO_CHECK < 5)
      {
        BLUE_COLOR = BLUE_COLOR - MODULO_CHECK;   /* Round to next lower number */       
      }
      else
      {
        /* Round to next higher number */
        BLUE_COLOR = BLUE_COLOR + (9-MODULO_CHECK);       
      }  
      MAX_RED_COLOR = RED_COLOR;                  /* Calculate max red color value */
      MAX_GREEN_COLOR = GREEN_COLOR;              /* Calculate max green color value */
      MAX_BLUE_COLOR = BLUE_COLOR;                /* Calculate max blue color value */
      RED_DIM = MAX_RED_COLOR/9;                  /* Set red dimmer value */
      GREEN_DIM = MAX_GREEN_COLOR/9;              /* Set green dimmer value */
      BLUE_DIM = MAX_BLUE_COLOR/9;                /* Set blue dimmer value */ 
      /* SELECT new color */   
      SUB_DMX_SEND (RED_COLOR, GREEN_COLOR, BLUE_COLOR);             
      UDP_COLOR = true;                           /* Set UDP color true */  
      SELECT = 255;                               /* Set command to 255 */
    }   
    delay(10);                                    /* 10ms Delay */                    
  }
}


/****************************************************************************************/
/* Subroutine to decode client request                                                  */
/* Decodes the message string and return a function code to display different results:  */
/* Result code 0 = Send OK message                                                      */
/* Result code 1 = Access denied error                                                  */
/* Result code 2 = Command error message                                                */
/* Result code 3 = Parameter error                                                      */
/* Result code 4 = Send RECEIVE_BUFEFR                                                  */
/****************************************************************************************/
byte SUB_DECODE_MESSAGE()
{  
  byte TEMP_BYTE;                                 /* Temporary byte for handling of strings */
  byte MENU_POS;                                  /* Actual menu position */
  int  TEST_INT;                                  /* Test Integer value for string conversion*/
  byte TEST_BYTE;                                 /* Test Byte value for string conversation */
  
  #ifdef _DEBUG_
    Serial.print ("  Received message: ");
    Serial.println (RECEIVE_BUFFER); 
  #endif 
  SUB_TRIGGER_WATCHDOG ();                        /* Trigger watchdog */ 

  /* Step 1: Decode login and PW */
  TEMP_BUFFER = RECEIVE_BUFFER.substring (0, 15);  
  #ifdef _DEBUG_ 
    Serial.print ("    Login and password: ");
    Serial.println (TEMP_BUFFER); 
  #endif    
  if (TEMP_BUFFER != LOGIN)                       /* Extracted substring is equal to login and password ? */     
  {
    #ifdef _DEBUG_
      Serial.println ("  Access denied error!!!");
      Serial.println ();
    #endif 
    return (1);                                   /* Set answer to 1 = Access denied error */  
  }  

  /*Step 2: Extract and handle command */
  TEMP_BUFFER = RECEIVE_BUFFER.substring (15, 17);  
  #ifdef _DEBUG_ 
    Serial.print ("    Command: ");
    Serial.println (TEMP_BUFFER); 
  #endif 
  
  /**************************************************************************************/
  /*                       Command handler                                              */
  /**************************************************************************************/
   /* Command is command ? */
  if (TEMP_BUFFER == "c=")                        /* Command c */
  {    
    TEMP_BUFFER = RECEIVE_BUFFER.substring (17, 20); 
    SELECT = TEMP_BUFFER.toInt();                 /* Convert into integer */
     #ifdef _DEBUG_ 
      Serial.print ("Received command c=");
      Serial.print (SELECT); 
    #endif    
    if ((SELECT < 1) || SELECT > 255)             /* No correct value ? */
    {
      #ifdef _DEBUG_
        Serial.println (" --> Answer Parameter Error!!!");
        Serial.println ();
      #endif  
      return (3);                                 /* Set answer to 3 = Parameter error */  
    } 
    else                                          /* In range ? */
    {        
      /****************************************************************/
      /* Program 141 set automatic mode off selected                  */
      /****************************************************************/
      if (SELECT == 141)                          /* Command automatic mode off ? */
      {
        #ifdef _DEBUG_
          Serial.println ("");
          Serial.println ("Setting device to Auto Power On mode = Off ... ");
        #endif   
        EEPROM.write(4,0);                        /* Storage position 4: Write Off = 0 to FLASH  */
        EEPROM.commit();                          /* Store values permanently in FLASH */
        SELECT = 255;                             /* Set command to 255 */ 
        return (0);                               /* Set answer to 0 = Send OK message */ 
      }

      /****************************************************************/
      /* Program 142 set autonatic mode on selected                   */
      /****************************************************************/
      if (SELECT == 142)                          /* Command automatic mode auto ? */
      {
        #ifdef _DEBUG_
          Serial.println ("");
          Serial.println ("Setting device to Auto Power On mode = Auto ... "); 
        #endif    
        EEPROM.write(4,1);                        /* Storage position 4: Write Auto = 1 to FLASH  */
        EEPROM.commit();                          /* Store values permanently in FLASH */  
        SELECT = 255;                             /* Set command to 255 */
        return (0);                               /* Set answer to 0 = Send OK message */ 
      }

      /****************************************************************/
      /* Program 144 set automatic mode off for slave 1 selected      */
      /****************************************************************/
      if (SELECT == 144)                          /* Command mode off for slave 1 ? */
      {
        #ifdef _DEBUG_
          Serial.println ("");
          Serial.println ("Setting slave 1 to Auto Power On mode = Off ... ");
        #endif  
        if (SLAVE_1_MODE !=0)                     /* Changed mode ? */
        {
          SLAVE_1_MODE = 0;                       /* Set slave 1 mode to 0 */
          EEPROM.write(5, 0);                     /* Store mode slave 1 in FLASH */
          EEPROM.commit();                        /* Write to FLASH memory */
          SLAVE_1_CHANGED = true;                 /* Set slave 1 mode changed via web interface true = Change */ 
        }      
        SELECT = 255;                             /* Set command to 255 */
        return (0);                               /* Set answer to 0 = Send OK message */ 
      }
    
      /****************************************************************/
      /* Program 145 set autonatic mode for slave 1 on selected       */
      /****************************************************************/
      if (SELECT == 145)                          /* Command mode on for slave 1 ? */
      {
        #ifdef _DEBUG_
          Serial.println ("");
          Serial.println ("Setting slave 1 to Auto Power On mode = On ... "); 
        #endif 
        if (SLAVE_1_MODE !=1)                     /* Changed mode ? */
        { 
          SLAVE_1_MODE = 1;                       /* Set slave 1 mode to 1 */
          EEPROM.write(5, 1);                     /* Store mode slave 1 in FLASH */
          EEPROM.commit();                        /* Write to FLASH memory */
          SLAVE_1_CHANGED = true;                 /* Set slave 1 mode changed via web interface true = Change */ 
        }
        SELECT = 255;                             /* Set command to 255 */
        return (0);                               /* Set answer to 0 = Send OK message */ 
      }
         
      /****************************************************************/
      /* Program 146 set automatic mode off for slave 2 selected      */
      /****************************************************************/
      if (SELECT == 146)                          /* Command mode off for slave 2 ? */
      {
        #ifdef _DEBUG_
          Serial.println ("");
          Serial.println ("Setting slave 2 to Auto Power On mode = Off ... ");
        #endif 
        if (SLAVE_2_MODE !=0)                     /* Changed mode ? */
        {
          SLAVE_2_MODE = 0;                       /* Set slave 2 mode to 0 */
          EEPROM.write(7, 0);                     /* Store mode slave 2 in FLASH */
          EEPROM.commit();                        /* Write to FLASH memory */
          SLAVE_2_CHANGED = true;                 /* Set slave 2 mode changed via web interface true = Change */ 
        }
        SELECT = 255;                             /* Set command to 255 */
        return (0);                               /* Set answer to 0 = Send OK message */ 
      }

      /****************************************************************/
      /* Program 147 set autonatic mode for slave 2 on selected       */
      /****************************************************************/
      if (SELECT == 147)                          /* Command mode on for slave 2 ? */
      {
        #ifdef _DEBUG_
          Serial.println ("");
          Serial.println ("Setting slave 2 to Auto Power On mode = On ... "); 
        #endif 
        if (SLAVE_2_MODE !=1)                     /* Changed mode ? */
        {          
          SLAVE_2_MODE = 1;                       /* Set slave 2 mode to 1 */
          EEPROM.write(7, 1);                     /* Store mode slave 2 in FLASH */
          EEPROM.commit();                        /* Write to FLASH memory */
          SLAVE_2_CHANGED = true;                 /* Set slave 2 mode changed via web interface true = Change */ 
        }
        SELECT = 255;                             /* Set command to 255 */
        return (0);                               /* Set answer to 0 = Send OK message */ 
      }

      /****************************************************************/
      /* Program 148 set automatic mode off for slave 3 selected      */
      /****************************************************************/
      if (SELECT == 148)                          /* Command Programe mode off for slave 3 ? */
      {
        #ifdef _DEBUG_
          Serial.println ("");
          Serial.println ("Setting slave 3 to Auto Power On mode = Off ... ");
        #endif 
        if (SLAVE_3_MODE !=0)                     /* Changed mode ? */
        {
          SLAVE_3_MODE = 0;                       /* Set slave 3 mode to 0 */
          EEPROM.write(9, 0);                     /* Store mode slave 3 in FLASH */
          EEPROM.commit();                        /* Write to FLASH memory */
          SLAVE_3_CHANGED = true;                 /* Set slave 3 mode changed via web interface true = Change */
        } 
        SELECT = 255;                             /* Set command to 255 */
        return (0);                               /* Set answer to 0 = Send OK message */ 
      }

      /****************************************************************/
      /* Program 149 set autonatic mode for slave 3 on selected       */
      /****************************************************************/
      if (SELECT == 149)                          /* Command mode on for slave 3 ? */
      {
        #ifdef _DEBUG_
          Serial.println ("");
          Serial.println ("Setting slave 3 to Auto Power On mode = On ... ");
        #endif   
        if (SLAVE_3_MODE !=1)                     /* Changed mode ? */
        {
          SLAVE_3_MODE = 1;                       /* Set slave 3 mode to 1 */
          EEPROM.write(9, 1);                     /* Store mode slave 3 in FLASH */
          EEPROM.commit();                        /* Write to FLASH memory */
          SLAVE_3_CHANGED = true;                 /* Set slave 3 mode changed via web interface true = Change */ 
        }
        SELECT = 255;                             /* Set command to 255 */
        return (0);                               /* Set answer to 0 = Send OK message */ 
      }
      
      /****************************************************************/
      /* Program 150 to enter bootloader                              */
      /****************************************************************/
      if (SELECT == 150)                          /* Command enter bootloader ? */
      {
        #ifdef _DEBUG_
          Serial.println ("");
          Serial.println ("Bootloader selected ... "); 
        #endif  
        BOOTLOADER_FLAG = true;                   /* Set bootloader flag true */
        SELECT = 255;                             /* Set command to 255 */
        return (0);                               /* Set answer to 0 = Send OK message */ 
      }    
      /****************************************************************/
      /* Program 151 to reset device                                  */
      /****************************************************************/
      if (SELECT == 151)                          /* Command to reset device ? */
      {
        #ifdef _DEBUG_
          Serial.println ("");
          Serial.println ("Reset selected ... ");
        #endif      
        RESET_FLAG = true;                        /* Set reset flag true */
        SELECT = 255;                             /* Set command to 255 */
        return (0);                               /* Set answer to 0 = Send OK message */ 
      }   

      /****************************************************************/
      /* Program 133 special command status request                   */
      /****************************************************************/
      if (SELECT == 133)                          /* Special command status request received ? --> Specific answer */
      {
        Serial.println ("Special command on status request received ...");
        /* a: Power On/Off Status: On/Off */
        if (POWER_OFF == false)                   /* Device in power On state? */
        {
          RECEIVE_BUFFER = "On";
        }
        else                                      /* Device in power Off state? */
        {
          RECEIVE_BUFFER = "Off";
        }   
        RECEIVE_BUFFER +=";";
        /*b: Actual time/date: XX:XX, XX.XX.20XX */
        if (ACT_HOUR < 10)                        /* Need to add leading zero ? */
        {
          RECEIVE_BUFFER +="0";                   /* Add leading zero */
        }
        RECEIVE_BUFFER += ACT_HOUR;               /* Add actual hour */
        RECEIVE_BUFFER +=":";
        if (ACT_MIN < 10)                         /* Need to add leading zero ? */
        {
          RECEIVE_BUFFER +="0";                   /* Add leading zero */
        }
        RECEIVE_BUFFER += ACT_MIN;                /* Add actual minute */
        RECEIVE_BUFFER +=", ";
        if (ACT_DAY < 10)                         /* Need to add leading zero ? */
        {
          RECEIVE_BUFFER +="0";                   /* Add leading zero */
        }            
        RECEIVE_BUFFER += ACT_DAY;                /* Add actual day */
        RECEIVE_BUFFER +=".";
        if (ACT_MONTH < 10)                       /* Need to add leading zero ? */
        {
          RECEIVE_BUFFER +="0";                   /* Add leading zero */
        }            
        RECEIVE_BUFFER += ACT_MONTH;              /* Add actual day */
        RECEIVE_BUFFER +=".";        
        RECEIVE_BUFFER += ACT_YEAR;               /* Add actual year */
        RECEIVE_BUFFER +=";";
        /* c: Actual On Time: XX:XX */
        if (ON_TIME_HOUR < 10)                    /* Need to add leading zero ? */
        {
          RECEIVE_BUFFER +="0";                   /* Add leading zero */
        }            
        RECEIVE_BUFFER += ON_TIME_HOUR;           /* Add on time hour */
        RECEIVE_BUFFER +=":";
        if (ON_TIME_MIN < 10)                     /* Need to add leading zero ? */
        {
          RECEIVE_BUFFER +="0";                   /* Add leading zero */
        }            
        RECEIVE_BUFFER += ON_TIME_MIN;            /* Add on time minute */
        RECEIVE_BUFFER +=";";
        /* d: Modus flag 0/1: Off/Auto */
        TEST_POS_1 = EEPROM.read (4);             /* Read mode bit from EEPROM */           
        if (TEST_POS_1 == 0)
        { 
          RECEIVE_BUFFER +="Off;";                /* Add Off information */
        }
        else
        { 
          RECEIVE_BUFFER +="Auto;";               /* Add Auto information */
        }  
        /* e: Slave 1 modus flag 0/1: Off/Auto */        
        if (SLAVE_1_MODE == 0)
        { 
          RECEIVE_BUFFER +="Off;";                /* Add Off information */
        }
        else
        { 
          RECEIVE_BUFFER +="On;";                 /* Add On information */
        } 
        /* f: Slave 2 modus flag 0/1: Off/Auto */          
        if (SLAVE_2_MODE == 0)
        { 
          RECEIVE_BUFFER +="Off;";                /* Add Off information */
        }
        else
        {   
          RECEIVE_BUFFER +="On;";                 /* Add On information */
        } 
        /* g: Slave 3 modus flag 0/1: Off/Auto */         
        if (SLAVE_3_MODE == 0)
        { 
          RECEIVE_BUFFER +="Off";                 /* Add Off information */
        }
        else
        { 
          RECEIVE_BUFFER +="On";                  /* Add On information */
        } 
        SELECT = 255;                             /* Set command to 255 */
        return (4);                               /* Set answer to 4 = Send RECEIVE_BUFEFR  */ 
      }

      /* Check on rest of valid commands */
      if (((SELECT>=1) && (SELECT <=13)) || ((SELECT>=15) && (SELECT <=29)) || ((SELECT>=32) && (SELECT <=40)) || ((SELECT>=48) && (SELECT <=51))|| ((SELECT>=64) && (SELECT <=73)))
      {
        return (0);                               /* Set answer to 0 = Send OK message */ 
      }   
      else                                        /* Error ? */
      {
        if (((SELECT>=128) && (SELECT <=131)) || ((SELECT>=136) && (SELECT <=151)))
        {
          return (0);                             /* Set answer to 0 = Send OK message */  
        }
        else                                      /* Error ? */
        {
          delay (100);                            /* 100ms delay */
          SELECT = 255;                           /* Set command to 255 */
          return (3);                             /* Set answer to 3 = Parameter error */  
        }
      }
    }
  }
  else                                            /* Command error? */
  {
    return (2);                                   /* Set answer to 2 = ommand error */ 
  }              
}


/****************************************************************************************/
/* Subroutine to define the routes for ESPAsyncWebserver                                */
/****************************************************************************************/
void SUB_DEFINE_ROUTES (void)
{                      
  /******************* Routes for requests starting with /?  ***************************/
  server.on("/", HTTP_GET, [] (AsyncWebServerRequest *request) 
  {
    byte ANSWER_FLAG;                             /* Answer from decode message subroutine. Needs to be defined in function! */
                                
    #ifdef _DEBUG_
        Serial.println ();               
        Serial.println("New client request...");
    #endif 
    if (request->hasParam(KEYWORD))               /* Keyword for single command detected ? */
    {
      /* Save rest of message in receive buffer */
      RECEIVE_BUFFER = request->getParam(KEYWORD)->value();
      ANSWER_FLAG = SUB_DECODE_MESSAGE();         /* Call subroutine to decode message */
      switch (ANSWER_FLAG)                        /* Send correct response based on returned answer from subroutine */
      {
        case 0:                                   /* 0 = Send OK message */
          request->send(200, "text/plain", "OK"); 
          break;
        case 1:                                   /* 1 = Access denied error */
          request->send(404, "text/plain", "Access denied error"); 
          break;          
        case 2:                                   /* 2 = Command error */
          request->send(404, "text/plain", "Command error"); 
          break;
        case 3:                                   /* 3 = Parameter error */
          request->send(404, "text/plain", "Parameter error"); 
          break;            
        case 4:                                   /* 4 = Send receive buffer */
          /* Needs to be be this way to be accepted by Mediola !!! */       
          delay (300);                            /* 300ms delay */
          request->send(200, "text/html", RECEIVE_BUFFER); 
          delay (300);                            /* 300ms delay */
          break;  
      }
    } 
    else                                          /* No keyword detected ? */ 
    {
      #ifdef _DEBUG_
        Serial.print ("  Message error!!! Keyword ");
        Serial.print (KEYWORD);
        Serial.println (" not detected!");
        Serial.println ();
      #endif  
      request->send(404, "text/plain", "Message error");
    }
  });

  /********************* Routes for loading CSS style sheet  ***************************/
  server.on("/style.css", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    /* Answer request by opening the style sheet */
    request->send(SPIFFS, "/Style.css", "text/css");
  });  

  /**************************** Routes for main page **********************************/
  /* Main menu: Load web page main.html */
  server.on("/main", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();               
      Serial.println ("Client connected: Display main page...");
      Serial.println ();
    #endif  
    /* Answer request with displaying of main web page */
    request->send(SPIFFS, "/Main.html", String(), false, SUB_WEB_PROCESSOR_MAIN);
  });

  /* Main menu: Button for device on/off pressed */
  server.on("/device", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ("Main menu: Button for device on/off pressed...");
      Serial.println ();
    #endif  
    if (POWER_OFF == true)                        /* Device in power off state ? */
    {
      SELECT = 129;                               /* Set selct code to switch device on */  
      POWER_WEB_OFF = false;                      /* Set power web off false */                              
    }
    else                                          /* Device in power on state ? */
    {
      SELECT = 128;                               /* Set selct code to switch device off */ 
      POWER_WEB_OFF = true;                       /* Set power web off true */ 
    }
    /* Answer request by refreshing main page */
    request->send(SPIFFS, "/Main.html", String(), false, SUB_WEB_PROCESSOR_MAIN);
  });

  /* Main menu: Refresh button pressed */
  server.on("/refreshm", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ("Main menu: Button for refresh of page pressed...");
      Serial.println ();
    #endif 
    /* Answer request by refreshing main page */
    request->send(SPIFFS, "/Main.html", String(), false, SUB_WEB_PROCESSOR_MAIN);
  });

  /************************** Routes for submenu page 1 *******************************/
   server.on("/submenu1", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();               
      Serial.println ("Client connected: Display submenu 1 page...");
      Serial.println ();
    #endif  
    /* Answer request with displaying of submain page 1 web page */
    request->send(SPIFFS, "/Submenu1.html", String(), false, SUB_WEB_PROCESSOR_SUBM1);
  }); 

  /* Submenu 1: Button for saving of the values pressed */
  server.on("/store1", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */
        
    #ifdef _DEBUG_
      Serial.println ("Submenu 1: Button for saving of values pressed...");
      Serial.println ();
    #endif 
    if (request->hasParam("SUB1MODE"))            /* Variable mode available? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB1MODE")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Mode set to: ");
        Serial.print (TEMP_INT);
        Serial.println (" (0 = Off, 1 = Auto)");
      #endif 
      EEPROM.write(4, TEMP_INT );                 /* Store mode in FLASH */
      EEPROM.commit();                            /* Write to FLASH memory */  
    } 

    if (request->hasParam("SUB1RIGHT"))            /* Variable mode slave 1 available */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB1RIGHT")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Mode slave 1 set to: ");
        Serial.print (TEMP_INT);
        Serial.println (" (0 = Off, 1 = On)");
      #endif 
      if (SLAVE_1_MODE != TEMP_INT)               /* Value has changed ? */
      {
        SLAVE_1_MODE = TEMP_INT;                  /* Save slave 1 mode */
        EEPROM.write(5, SLAVE_1_MODE );           /* Store mode slave 1 in FLASH */
        EEPROM.commit();                          /* Write to FLASH memory */
        SLAVE_1_CHANGED = true;                   /* Set slave 1 mode changed via web interface true = Change */ 
      }
    } 

    if (request->hasParam("SUB1LEFT"))           /* Variable mode slave 2 available */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB1LEFT")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Mode slave 2 set to: ");
        Serial.print (TEMP_INT);
        Serial.println (" (0 = Off, 1 = On)");
      #endif 
      if (SLAVE_2_MODE != TEMP_INT)               /* Value has changed ? */
      {
        SLAVE_2_MODE = TEMP_INT;                  /* Save slave 2 mode */
        EEPROM.write(7, SLAVE_2_MODE );           /* Store mode slave 2 in FLASH */
        EEPROM.commit();                          /* Write to FLASH memory */
        SLAVE_2_CHANGED = true;                   /* Set slave 2 mode changed via web interface true = Change */ 
      }
    } 

    if (request->hasParam("SUB1EG"))              /* Variable mode slave 3 available */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB1EG")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Mode slave 3 set to: ");
        Serial.print (TEMP_INT);
        Serial.println (" (0 = Off, 1 = On)");
        Serial.println ();
      #endif 
      if (SLAVE_3_MODE != TEMP_INT)               /* Value has changed ? */
      {
        SLAVE_3_MODE = TEMP_INT;                  /* Save slave 3 mode */
        EEPROM.write(9, SLAVE_3_MODE );           /* Store mode slave 3 in FLASH */
        EEPROM.commit();                          /* Write to FLASH memory */
        SLAVE_3_CHANGED = true;                   /* Set slave 3 mode changed via web interface true = Change */ 
      }
    } 
    
    if (request->hasParam("SUB1NTPHOUR"))         /* Variable NTP hour available? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB1NTPHOUR")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      if (NTP_UPDATE_HOUR  != TEMP_INT)           /* Value has changed ? */
      {
        NTP_UPDATE_HOUR = TEMP_INT;               /* Save new NTP update hour */               
        #ifdef _DEBUG_
          Serial.print ("  NTP Update hour set to: ");
          Serial.println (NTP_UPDATE_HOUR);
          Serial.println ();
        #endif 
        EEPROM.write(61, NTP_UPDATE_HOUR );       /* Store NTP update hour in FLASH */
        EEPROM.commit();                          /* Write to FLASH memory */   
      }     
    }     
    
    if (request->hasParam("SUB1NTPMIN"))          /* Variable NTP minute available? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB1NTPMIN")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      if (NTP_UPDATE_MIN  != TEMP_INT)            /* Value has changed ? */
      {
        NTP_UPDATE_MIN = TEMP_INT;                /* Save new NTP update minute */               
        #ifdef _DEBUG_
          Serial.print ("  NTP Update minute set to: ");
          Serial.println (NTP_UPDATE_MIN);
          Serial.println ();
        #endif 
        EEPROM.write(62, NTP_UPDATE_MIN );        /* Store NTP update minute in FLASH */
        EEPROM.commit();                          /* Write to FLASH memory */   
      }     
    }  
    
    if (request->hasParam("SUB1NTPZONE"))         /* Variable NTP zone available? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB1NTPZONE")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      if (NTP_TIME_ZONE  != TEMP_INT)             /* Value has changed ? */
      {
        NTP_TIME_ZONE = TEMP_INT;                 /* Save new NTP zone value */               
        #ifdef _DEBUG_
          Serial.print ("  NTP zone set to: ");
          switch (NTP_TIME_ZONE)                  /* Display time zone on NEXTION display*/
          {
            case 0:                               /* 0 = - 5 hours */       
              Serial.println ("-5 hours");
              break;    
            case 1:                               /* 1 = - 4 hours */       
              Serial.println ("-4 hours");
              break;    
            case 2:                               /* 2 = - 3 hours */       
              Serial.println ("-3 hours");
              break; 
            case 3:                               /* 3 = - 2 hours */       
              Serial.println ("-2 hours");
              break; 
            case 4:                               /* 4 = - 1 hour */       
              Serial.println ("-1 hour");
              break; 
            case 5:                               /* 5 = 0 hours */       
              Serial.println ("0 hours");
              break; 
            case 6:                               /* 6 = 1 hour */       
              Serial.println ("+1 hour");
              break; 
            case 7:                               /* 7 = + 2 hours */       
              Serial.println ("+2 hours");
              break;               
            case 8:                               /* 8 = + 3 hours */       
              Serial.println ("+3 hours");
              break;
            case 9:                               /* 9 = + 4 hours */       
              Serial.println ("+4 hours");
              break;
            case 10:                              /* 10 = + 5 hours */       
              Serial.println ("+5 hours");
              break;                                              
          }
          Serial.println ();
        #endif 
        EEPROM.write(63, NTP_TIME_ZONE);          /* Store NTP update minute in FLASH */
        EEPROM.commit();                          /* Write to FLASH memory */ 
        /* Calculated from NTP_TIME_ZONE, e.g. Austia = UTC + 1 hour = 3600 seconds */
        GMT_OFFSET_SEC = NTP_TIME_ZONE_VALUE[NTP_TIME_ZONE]*3600; 
        SUB_NTP_RECEIVE();                        /* Call subroutine to receive NTP time */          
      }
    } 

    if (request->hasParam("SUB1NTPSOM"))          /* Variable NTP automatic summertime switching ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB1NTPSOM")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      /* Value has changed ? */
      /* TEMP_INT: 1 = No summertime sync, 2 = Summertime sync */
      if (((TEMP_INT == 1) && (NTP_SUMMERTIME == true)) || ((TEMP_INT == 2) && (NTP_SUMMERTIME == false)))
      {
        if (TEMP_INT == 1)                        /* No summertime switching ? */
        { 
          NTP_SUMMERTIME = false;                 /* Set NTP summertime switching flag false */
          EEPROM.write(64, 0);                    /* Store NTP summertime switching status */
          EEPROM.commit();                        /* Write to FLASH memory */  
          #ifdef _DEBUG_
            Serial.print ("  NTP automatic summertime switching: Disabled");  
          #endif                   
        }
        else
        {
          NTP_SUMMERTIME = true;                  /* Set NTP summertime switching flag true */
          EEPROM.write(64, 1);                    /* Store NTP summertime switching status */
          EEPROM.commit();                        /* Write to FLASH memory */ 
          #ifdef _DEBUG_
            Serial.print ("  NTP automatic summertime switching: Enabled");  
          #endif           
        }
        SUB_NTP_RECEIVE();                        /* Call subroutine to receive NTP time */          
      }    
    }   
    /* Answer request with displaying of submain page 1 web page */
    request->send(SPIFFS, "/Submenu1.html", String(), false, SUB_WEB_PROCESSOR_SUBM1);
  }); 

  /************************** Routes for submenu page 2 *******************************/
   server.on("/submenu2", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();               
      Serial.println ("Client connected: Display submenu 2 page...");
      Serial.println ();
    #endif  
    /* Answer request with displaying of submain page 2 web page */
    request->send(SPIFFS, "/Submenu2.html", String(), false, SUB_WEB_PROCESSOR_SUBM2);
  }); 

  /* Submenu 2: Button for saving of the values pressed */
  server.on("/store2", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */
        
    #ifdef _DEBUG_
      Serial.println ("Submenu 2: Button for saving of values pressed...");
      Serial.println ();
    #endif 
    if (request->hasParam("SUB2COLOR"))           /* Variable single color available? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB2COLOR")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Color set to: ");
        switch (TEMP_INT)
        {
          case 0:
            Serial.println ("---");
            break;
          case 1:
            Serial.println ("Cold white");
            break;
          case 2:
            Serial.println ("Warm white");
            break;
          case 3:
            Serial.println ("Yellow");
            break;
          case 4:
            Serial.println ("Orange");
            break;
          case 5:
            Serial.println ("Red");
            break;
          case 6:
            Serial.println ("Dark purple");
            break; 
          case 7:
            Serial.println ("Purple");
            break;
          case 8:
            Serial.println ("Bright purple");
            break; 
          case 9:
            Serial.println ("Blue");
            break; 
          case 10:
            Serial.println ("Bright blue");
            break; 
          case 11:
            Serial.println ("Cyan");
            break; 
          case 12:
            Serial.println ("Bright green");
            break; 
          case 13:
            Serial.println ("Green");
            break;
        }                        
      #endif 
      if ((TEMP_INT >=1)&& (TEMP_INT <= 13))      /* In range? */  
      {
        SELECT = TEMP_INT;    
        ACT_PROGRAM = SELECT;                     /* Save in actual program */         
      }
    } 

    if (request->hasParam("SUB2DIM"))             /* Variable single color brightness available? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB2DIM")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Brightness for color set to: ");
        switch (TEMP_INT)
        {
          case 0:
            Serial.println ("100%");
            break; 
          case 1:
            Serial.println ("85%");
            break; 
          case 2:
            Serial.println ("70%");
            break; 
          case 3:
            Serial.println ("60%");
            break; 
          case 4:
            Serial.println ("50%");
            break; 
          case 5:
            Serial.println ("40%");
            break; 
          case 6:
            Serial.println ("30%");
            break;                                     
          case 7:
            Serial.println ("20%");
            break; 
          case 8:
            Serial.println ("10%");
            break; 
        }                        
      #endif 
      COLOR_DIM_STEP = TEMP_INT;                  /* Save diming factor for fixed color */
      EEPROM.write(1,COLOR_DIM_STEP);             /* Save diming factor for fixed color in FLASH  */
      EEPROM.commit();                            /* Store values permanently in FLASH */         
    }    
    /* Answer request with displaying of submain page 2 web page */
    request->send(SPIFFS, "/Submenu2.html", String(), false, SUB_WEB_PROCESSOR_SUBM2);
  }); 

  /************************** Routes for submenu page 3 *******************************/
   server.on("/submenu3", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();               
      Serial.println ("Client connected: Display submenu 3 page...");
      Serial.println ();
    #endif  
    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
  }); 

  /* Submenu 3: Button for saving of the values pressed */
  server.on("/store3", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */
        
    #ifdef _DEBUG_
      Serial.println ("Submenu 3: Button for saving of values pressed...");
      Serial.println ();
    #endif 
    if (request->hasParam("SUB3PROG"))            /* Variable program available? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3PROG")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Program set to: ");
        switch (TEMP_INT)
        {          
          case 0:
            Serial.println ("---");
            break;
          case 1:
            Serial.println ("Farbwechsel 1");
            break;
          case 2:
            Serial.println ("Farbwechsel 2");
            break;
          case 3:
            Serial.println ("Farbwechsel 3");
            break;
          case 4:
            Serial.println ("Farbwechsel 4");
            break;
          case 5:
            Serial.println ("Farbwechsel 5");
            break;
          case 6:
            Serial.println ("Schritte");
            break; 
          case 7:
            Serial.println ("Farbenspiel 1");
            break;
          case 8:
            Serial.println ("Farbenspiel 2");
            break; 
          case 9:
            Serial.println ("Karminfeuer");
            break; 
          case 10:
            Serial.println ("Gewitter");
            break; 
        }                        
      #endif 
      if ((TEMP_INT >=1)&& (TEMP_INT <= 10))      /* In range? */  
      {
        SELECT = TEMP_INT + 19; 
        ACT_PROGRAM = SELECT;                     /* Save in actual program */           
      }
    } 

    if (request->hasParam("SUB3DIM"))             /* Variable program brightness available? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3DIM")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Brightness for program set to: ");
        switch (TEMP_INT)
        {
        
          case 0:
            Serial.println ("100%");
            break; 
          case 1:
            Serial.println ("75%");
            break; 
          case 2:
            Serial.println ("50%");
            break; 
          case 3:
            Serial.println ("25%");
            break; 
        }                        
      #endif 
      PROG_DIM_STEP = TEMP_INT;                   /* Save diming factor for program */
      EEPROM.write(2,PROG_DIM_STEP);              /* Save diming factor for program in FLASH  */
      EEPROM.commit();                            /* Store values permanently in FLASH */         
    }  

    if (request->hasParam("SUB3SPEED"))           /* Variable program speed available? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3SPEED")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Speed for program set to: ");
        switch (TEMP_INT)
        {
          case 0:
            Serial.println ("100%");
            break; 
          case 1:
            Serial.println ("90%");
            break; 
          case 2:
            Serial.println ("80%");
            break; 
          case 3:
            Serial.println ("70%");
            break; 
          case 4:
            Serial.println ("60%");
            break;        
          case 5:
            Serial.println ("50%");
            break;    
          case 6:
            Serial.println ("40%");
            break; 
          case 7:
            Serial.println ("30%");
            break; 
          case 8:
            Serial.println ("20%");
            break;    
          case 9:
            Serial.println ("10%");
            break;                                    
        }                        
      #endif 
      PROG_SPEED = TEMP_INT;                      /* Save speed factor for program */
      EEPROM.write(3,PROG_SPEED);                 /* Save diming factor for program in FLASH  */
      EEPROM.commit();                            /* Store values permanently in FLASH */         
    }

    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
  }); 

  /* Submenu 3: Button for pause pressed */
  server.on("/pause", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ("Submenu 3: Button for pause of program pressed...");
      Serial.println ();
    #endif    
    SELECT = 130;                                 /* Send code for pause */      
    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
  });

  /* Submenu 3: Button for play pressed */
  server.on("/play", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ("Submenu 3: Button for play of program pressed...");
      Serial.println ();
    #endif    
    SELECT = 131;                                 /* Send code for play */      
    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
  });

  /************************** Routes for submenu page 4 *******************************/
   server.on("/submenu4", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();               
      Serial.println ("Client connected: Display submenu 4 page...");
      Serial.println ();
    #endif  
    SUB_MENU_FLAG = false;                        /* Set menu flag false = Display zero */
    /* Answer request with displaying of submain page 4 web page */
    request->send(SPIFFS, "/Submenu4.html", String(), false, SUB_WEB_PROCESSOR_SUBM4);
  }); 

  /* Submenu 4: Button for saving of the values pressed */
  server.on("/store4", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */
        
    #ifdef _DEBUG_
      Serial.println ("Submenu 4: Button for saving of values pressed...");
      Serial.println ();
    #endif 
    if (request->hasParam("SUB4STORE"))           /* Variable program available? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4STORE")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Program set to: ");
        switch (TEMP_INT)
        {          
          case 0:
            Serial.println ("---");
            break;
          case 1:
            Serial.println ("Speicher 1");
            break;
          case 2:
            Serial.println ("Speicher 2");
            break;
          case 3:
            Serial.println ("Speicher 3");
            break;
          case 4:
            Serial.println ("Speicher 4");
            break;
          case 5:
            Serial.println ("Speicher 5");
            break;
        }                        
      #endif 
      if ((TEMP_INT >=1)&& (TEMP_INT <= 5))       /* In range? */  
      {
        SELECT = TEMP_INT + 14; 
        ACT_PROGRAM = SELECT;                     /* Save in actual program */           
      }
      SUB_MENU_FLAG = false;                      /* Set menu flag false = Display zero */
    } 

    /* Answer request with displaying of submain page 4 web page */
    request->send(SPIFFS, "/Submenu4.html", String(), false, SUB_WEB_PROCESSOR_SUBM4);
  }); 

  /* Submenu 4: Button for recalling storage 1 pressed */
  server.on("/rcl1", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ("Submenu 4: Button for recalling storage position 1 pressed...");
      Serial.println ();
    #endif     
    STORE_RED_COLOR   = EEPROM.read (10);         /* Red color value of storage position 1 */
    STORE_GREEN_COLOR = EEPROM.read (11);         /* Green color value of storage position 1 */
    STORE_BLUE_COLOR  = EEPROM.read (12);         /* Blue color value of storage position 1 */
    STORE_DIM_VALUE   = EEPROM.read (13);         /* Dimming value of storage position 1 */ 
    SELECT = 15;                                  /* Set program 1 */
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */

    /* Answer request with displaying of submain page 4A web page */
    request->send(SPIFFS, "/Submenu4.html", String(), false, SUB_WEB_PROCESSOR_SUBM4);
  });

  /* Submenu 4: Button for recalling storage 2 pressed */
  server.on("/rcl2", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ("Submenu 4: Button for recalling storage position 2 pressed...");
      Serial.println ();
    #endif       
    STORE_RED_COLOR   = EEPROM.read (20);         /* Red color value of storage position 2 */
    STORE_GREEN_COLOR = EEPROM.read (21);         /* Green color value of storage position 2 */
    STORE_BLUE_COLOR  = EEPROM.read (22);         /* Blue color value of storage position 2 */
    STORE_DIM_VALUE   = EEPROM.read (23);         /* Dimming value of storage position 2 */   
    SELECT = 16;                                  /* Set program 2 */
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */

    /* Answer request with displaying of submain page 4 web page */
    request->send(SPIFFS, "/Submenu4.html", String(), false, SUB_WEB_PROCESSOR_SUBM4);
  });

  /* Submenu 4: Button for recalling storage 3 pressed */
  server.on("/rcl3", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ("Submenu 4: Button for recalling storage position 3 pressed...");
      Serial.println ();
    #endif   
    STORE_RED_COLOR   = EEPROM.read (30);         /* Red color value of storage position 3 */
    STORE_GREEN_COLOR = EEPROM.read (31);         /* Green color value of storage position 3 */
    STORE_BLUE_COLOR  = EEPROM.read (32);         /* Blue color value of storage position 3 */
    STORE_DIM_VALUE   = EEPROM.read (33);         /* Dimming value of storage position 3 */ 
    SELECT = 17;                                  /* Set program 3 */  
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */

    /* Answer request with displaying of submain page 4 web page */
    request->send(SPIFFS, "/Submenu4.html", String(), false, SUB_WEB_PROCESSOR_SUBM4);
  });

  /* Submenu 4: Button for recalling storage 4 pressed */
  server.on("/rcl4", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ("Submenu 4: Button for recalling storage position 4 pressed...");
      Serial.println ();
    #endif       
    STORE_RED_COLOR   = EEPROM.read (40);         /* Red color value of storage position 4 */
    STORE_GREEN_COLOR = EEPROM.read (41);         /* Green color value of storage position 4 */
    STORE_BLUE_COLOR  = EEPROM.read (42);         /* Blue color value of storage position 4 */
    STORE_DIM_VALUE   = EEPROM.read (43);         /* Dimming value of storage position 4 */   
    SELECT = 18;                                  /* Set program 4 */
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */    

    /* Answer request with displaying of submain page 4 web page */
    request->send(SPIFFS, "/Submenu4.html", String(), false, SUB_WEB_PROCESSOR_SUBM4);
  });

  /* Submenu 4: Button for recalling storage 5 pressed */
  server.on("/rcl5", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ("Submenu 4: Button for recalling storage position 5 pressed...");
      Serial.println ();
    #endif       
    STORE_RED_COLOR   = EEPROM.read (50);         /* Red color value of storage position 5 */
    STORE_GREEN_COLOR = EEPROM.read (51);         /* Green color value of storage position 5 */
    STORE_BLUE_COLOR  = EEPROM.read (52);         /* Blue color value of storage position 5 */
    STORE_DIM_VALUE   = EEPROM.read (53);         /* Dimming value of storage position 5 */ 
    SELECT = 19;                                  /* Set program 5 */  
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */    

    /* Answer request with displaying of submain page 4 web page */
    request->send(SPIFFS, "/Submenu4.html", String(), false, SUB_WEB_PROCESSOR_SUBM4);
  });

  /* Submenu 4: Button for storing storage 1 pressed */
  server.on("/sto1", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */

    #ifdef _DEBUG_
      Serial.println ("Submenu 4: Button for writing storage position 1 pressed...");
      Serial.println ();
    #endif        
    if (request->hasParam("SUB4RED"))             /* Variable red value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4RED")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 1 red value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (10, TEMP_INT);                /* Write red value to storage position 1 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB4GREEN"))           /* Variable green value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4GREEN")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 1 green value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (11, TEMP_INT);                /* Write green value to storage position 1 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB4BLUE"))            /* Variable blue value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4BLUE")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 1 blue value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (12, TEMP_INT);                /* Write blue value to storage position 1 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }
        
    if (request->hasParam("SUB4DIM"))             /* Variable dim value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4DIM")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 1 dimming value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (13, TEMP_INT);                /* Write dimming value to storage position 1 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */      

    /* Answer request with displaying of submain page 4 web page */
    request->send(SPIFFS, "/Submenu4.html", String(), false, SUB_WEB_PROCESSOR_SUBM4);
  });

  /* Submenu 4: Button for storing storage 2 pressed */
  server.on("/sto2", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */
    
    #ifdef _DEBUG_
      Serial.println ("Submenu 4: Button for writing storage position 2 pressed...");
      Serial.println ();
    #endif          
    if (request->hasParam("SUB4RED"))             /* Variable red value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4RED")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 2 red value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (20, TEMP_INT);                /* Write red value to storage position 2 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB4GREEN"))           /* Variable green value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4GREEN")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 2 green value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (21, TEMP_INT);                /* Write green value to storage position 2 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB4BLUE"))            /* Variable blue value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4BLUE")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 2 blue value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (22, TEMP_INT);                /* Write blue value to storage position 2 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }
        
    if (request->hasParam("SUB4DIM"))             /* Variable dim value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4DIM")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 2 dimming value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (23, TEMP_INT);                /* Write dimming value to storage position 2 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }      
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */ 
    
    /* Answer request with displaying of submain page 4 web page */
    request->send(SPIFFS, "/Submenu4.html", String(), false, SUB_WEB_PROCESSOR_SUBM4);
  });

  /* Submenu 4: Button for storing storage 3 pressed */
  server.on("/sto3", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */

    #ifdef _DEBUG_
      Serial.println ("Submenu 4: Button for writing storage position 3 pressed...");
      Serial.println ();
    #endif          
    if (request->hasParam("SUB4RED"))             /* Variable red value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4RED")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 3 red value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (30, TEMP_INT);                /* Write red value to storage position 3 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB4GREEN"))           /* Variable green value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4GREEN")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 3 green value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (31, TEMP_INT);                /* Write green value to storage position 3 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB4BLUE"))            /* Variable blue value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4BLUE")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 3 blue value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (32, TEMP_INT);                /* Write blue value to storage position 3 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }
        
    if (request->hasParam("SUB4DIM"))             /* Variable dim value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4DIM")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 3 dimming value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (33, TEMP_INT);                /* Write dimming value to storage position 3 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */           

    /* Answer request with displaying of submain page 4 web page */
    request->send(SPIFFS, "/Submenu4.html", String(), false, SUB_WEB_PROCESSOR_SUBM4);
  });

  /* Submenu 4: Button for storing storage 4 pressed */
  server.on("/sto4", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */

    #ifdef _DEBUG_
      Serial.println ("Submenu 4: Button for writing storage position 4 pressed...");
      Serial.println ();
    #endif      
    if (request->hasParam("SUB4RED"))             /* Variable red value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4RED")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 4 red value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (40, TEMP_INT);                /* Write red value to storage position 4 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB4GREEN"))           /* Variable green value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4GREEN")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 4 green value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (41, TEMP_INT);                /* Write green value to storage position 4 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB4BLUE"))            /* Variable blue value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4BLUE")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 4 blue value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (42, TEMP_INT);                /* Write blue value to storage position 4 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }
        
    if (request->hasParam("SUB4DIM"))             /* Variable dim value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4DIM")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 4 dimming value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (43, TEMP_INT);                /* Write dimming value to storage position 4 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */          

    /* Answer request with displaying of submain page 4 web page */
    request->send(SPIFFS, "/Submenu4.html", String(), false, SUB_WEB_PROCESSOR_SUBM4);
  });


  /* Submenu 4: Button for storing storage 5 pressed */
  server.on("/sto5", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */
    
    #ifdef _DEBUG_
      Serial.println ("Submenu 4: Button for writing storage position 5 pressed...");
      Serial.println ();
    #endif          
    if (request->hasParam("SUB4RED"))             /* Variable red value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4RED")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 5 red value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (50, TEMP_INT);                /* Write red value to storage position 5 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB4GREEN"))           /* Variable green value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4GREEN")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 5 green value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (51, TEMP_INT);                /* Write green value to storage position 5 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB4BLUE"))            /* Variable blue value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4BLUE")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 5 blue value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (52, TEMP_INT);                /* Write blue value to storage position 5 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }
        
    if (request->hasParam("SUB4DIM"))             /* Variable dim value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4DIM")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 5 dimming value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (53, TEMP_INT);                /* Write dimming value to storage position 5 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }      
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */ 
    
    /* Answer request with displaying of submain page 4 web page */
    request->send(SPIFFS, "/Submenu4.html", String(), false, SUB_WEB_PROCESSOR_SUBM4);
  });

  /* Submenu 4: Button for test pressed */
  server.on("/test", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */

    #ifdef _DEBUG_
      Serial.println ("Submenu 4: Button for testing of colors pressed...");
      Serial.println ();
    #endif          
    if (request->hasParam("SUB4RED"))             /* Variable red value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4RED")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Test red value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      STORE_RED_COLOR = TEMP_INT;                 /* Save red value */ 
    }  
        
    if (request->hasParam("SUB4GREEN"))           /* Variable green value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4GREEN")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Test green value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      STORE_GREEN_COLOR = TEMP_INT;               /* Save green value */ 
    } 
        
    if (request->hasParam("SUB4BLUE"))            /* Variable blue value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4BLUE")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Test1 blue value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      STORE_BLUE_COLOR = TEMP_INT;                /* Save blue value */
    }
        
    if (request->hasParam("SUB4DIM"))             /* Variable dim value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB4DIM")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Test dimming value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      STORE_DIM_VALUE = TEMP_INT;                 /* Save blue value */
    }  

    TEST_COLOR = true;                            /* Set test color flag true */
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */ 

    /* Answer request with displaying of submain page 4 web page */
    request->send(SPIFFS, "/Submenu4.html", String(), false, SUB_WEB_PROCESSOR_SUBM4);
  });

  /* Submenu 4: Button for restore pressed */
  server.on("/restore", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ("Submenu 4: Button for restoring storage position pressed...");
      Serial.println ();
    #endif          
    /* Answer request with restore page */
    request->send(SPIFFS, "/Restore.html");
  });
  

  /************************** Routes for submenu page 5 *******************************/
   server.on("/submenu5", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();               
      Serial.println ("Client connected: Display submenu 5 page...");
      Serial.println ();
    #endif  
    /* Answer request with displaying of submain page 5 web page */
    request->send(SPIFFS, "/Submenu5.html", String(), false, SUB_WEB_PROCESSOR_SUBM5);
  }); 

  /* Submenu 5: Button for saving of the values pressed */
  server.on("/store5", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for length and other purposes */
    
    #ifdef _DEBUG_
      Serial.println ("Submenu 5: Button for saving of values pressed...");
      Serial.println ();
    #endif  
    if (request->hasParam("SUB5HOST"))            /* Variable for hostname ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5HOST")->value();
      TEMP_INT = TEMP_BUFFER.length();            /* Determine length of the string */
      /* Fill char array with string */
      TEMP_BUFFER.toCharArray(HOSTNAME, TEMP_INT+1);      
      #ifdef _DEBUG_
        Serial.print ("  Hostname set to: ");
        Serial.println (HOSTNAME);
      #endif       
    }  
    if (request->hasParam("SUB5IPA"))             /* Variable for fixed IP 4th digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5IPA")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      FIXED_IP[0] = TEMP_INT;                     /* Save 4th digit */  
    }  
    if (request->hasParam("SUB5IPB"))             /* Variable for fixed IP 3rd digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5IPB")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      FIXED_IP[1] = TEMP_INT;                     /* Save 3rd digit */  
    } 
    if (request->hasParam("SUB5IPC"))             /* Variable for fixed IP 2nd digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5IPC")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      FIXED_IP[2] = TEMP_INT;                     /* Save 2nd digit */  
    } 
    if (request->hasParam("SUB5IPD"))             /* Variable for fixed IP 1st digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5IPD")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      FIXED_IP[3] = TEMP_INT;                     /* Save 1st digit */  
      #ifdef _DEBUG_
        Serial.print ("  Fixed IP set to: ");
        Serial.println (FIXED_IP);
      #endif 
    } 
    if (request->hasParam("SUB5SUA"))             /* Variable for subnet mask 4th digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5SUA")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      SUBNET[0] = TEMP_INT;                       /* Save 4th digit */  
    }  
    if (request->hasParam("SUB5SUB"))             /* Variable for subnet mask 3rd digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5SUB")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      SUBNET[1] = TEMP_INT;                       /* Save 3rd digit */  
    } 
    if (request->hasParam("SUB5SUC"))             /* Variable for subnet mask 2nd digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5SUC")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      SUBNET[2] = TEMP_INT;                       /* Save 2nd digit */  
    } 
    if (request->hasParam("SUB5SUD"))             /* Variable for subnet mask 1st digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5SUD")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      SUBNET[3] = TEMP_INT;                       /* Save 1st digit */  
      #ifdef _DEBUG_
        Serial.print ("  Subnet mask set to: ");
        Serial.println (SUBNET);
      #endif 
    } 
    if (request->hasParam("SUB5GWA"))             /* Variable for gateway IP address 4th digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5GWA")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      GATEWAY[0] = TEMP_INT;                      /* Save 4th digit */  
    }  
    if (request->hasParam("SUB5GWB"))             /* Variable for gateway IP address 3rd digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5GWB")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      GATEWAY[1] = TEMP_INT;                      /* Save 3rd digit */  
    } 
    if (request->hasParam("SUB5GWC"))             /* Variable for gateway IP address 2nd digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5GWC")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      GATEWAY[2] = TEMP_INT;                      /* Save 2nd digit */  
    } 
    if (request->hasParam("SUB5GWD"))             /* Variable for gateway IP address 1st digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5GWD")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      GATEWAY[3] = TEMP_INT;                      /* Save 1st digit */  
      #ifdef _DEBUG_
        Serial.print ("  Gateway IP address set to: ");
        Serial.println (GATEWAY);
      #endif 
    } 
    if (request->hasParam("SUB5D1A"))             /* Variable for DNS 1 IP address 4th digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5D1A")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      DNS_1[0] = TEMP_INT;                        /* Save 4th digit */  
    }  
    if (request->hasParam("SUB5D1B"))             /* Variable for DNS 1 IP address 3rd digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5D1B")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      DNS_1[1] = TEMP_INT;                        /* Save 3rd digit */  
    } 
    if (request->hasParam("SUB5D1C"))             /* Variable for DNS 1 IP address 2nd digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5D1C")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      DNS_1[2] = TEMP_INT;                        /* Save 2nd digit */  
    } 
    if (request->hasParam("SUB5D1D"))             /* Variable for DNS 1 IP address 1st digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5D1D")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      DNS_1[3] = TEMP_INT;                        /* Save 1st digit */  
      #ifdef _DEBUG_
        Serial.print ("  DNS 1 IP address set to: ");
        Serial.println (DNS_1);
      #endif 
    } 
    if (request->hasParam("SUB5D2A"))             /* Variable for DNS 2 IP address 4th digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5D2A")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      DNS_2[0] = TEMP_INT;                        /* Save 4th digit */  
    }  
    if (request->hasParam("SUB5D2B"))             /* Variable for DNS 2 IP address 3rd digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5D2B")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      DNS_2[1] = TEMP_INT;                        /* Save 3rd digit */  
    } 
    if (request->hasParam("SUB5D2C"))             /* Variable for DNS 2 IP address 2nd digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5D2C")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      DNS_2[2] = TEMP_INT;                        /* Save 2nd digit */  
    } 
    if (request->hasParam("SUB5D2D"))             /* Variable for DNS 2 IP address 1st digit ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5D2D")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      DNS_2[3] = TEMP_INT;                        /* Save 1st digit */  
      #ifdef _DEBUG_
        Serial.print ("  DNS 2 IP address set to: ");
        Serial.println (DNS_2);
      #endif 
    }   
    if (request->hasParam("SUB5NTP1"))            /* Variable for NTP 1 server ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5NTP1")->value();
      TEMP_INT = TEMP_BUFFER.length();            /* Determine length of the string */
      /* Fill char array with string */
      TEMP_BUFFER.toCharArray(NTP_SERVER_1, TEMP_INT+1);      
      #ifdef _DEBUG_
        Serial.print ("  NTP 1 server name set to: ");
        Serial.println (NTP_SERVER_1);
      #endif       
    }
    if (request->hasParam("SUB5NTP2"))            /* Variable for NTP 2 server ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5NTP2")->value();
      TEMP_INT = TEMP_BUFFER.length();            /* Determine length of the string */
      /* Fill char array with string */
      TEMP_BUFFER.toCharArray(NTP_SERVER_2, TEMP_INT+1);      
      #ifdef _DEBUG_
        Serial.print ("  NTP 2 server name set to: ");
        Serial.println (NTP_SERVER_2);
      #endif       
    }
    if (request->hasParam("SUB5NTP3"))            /* Variable for NTP 3 server ? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB5NTP3")->value();
      TEMP_INT = TEMP_BUFFER.length();            /* Determine length of the string */
      /* Fill char array with string */
      TEMP_BUFFER.toCharArray(NTP_SERVER_3, TEMP_INT+1);      
      #ifdef _DEBUG_
        Serial.print ("  NTP 3 server name set to: ");
        Serial.println (NTP_SERVER_3);
        Serial.println ();
      #endif       
    }
    RESET_FLAG = true;                            /* Set reset flag true */
    request->send(SPIFFS, "/Reboot.html");
  }); 

  /************************** Routes for submenu page 6 *******************************/
   server.on("/submenu6", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();               
      Serial.println ("Client connected: Display submenu 6 page...");
      Serial.println ();
    #endif  
    /* Answer request with displaying of submain page 6 web page */
    request->send(SPIFFS, "/Submenu6.html", String(), false, SUB_WEB_PROCESSOR_SUBM6);
  });   

  /* Submenu 6: Button for saving of the values pressed */
  server.on("/store6", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */
    
    #ifdef _DEBUG_
      Serial.println ("Submenu 6: Button for saving of bootloader status pressed...");
      Serial.println ();
    #endif 
    if (request->hasParam("SUB6BOOT"))            /* Variable for boot mode? */
    {  
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB6BOOT")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */              
      #ifdef _DEBUG_
        Serial.print ("  Boot mode set to: ");
        if (TEMP_INT == 1)                        /* Normal boot ? */
        {
          Serial.println ("Normal");
        }
        else                                      /* Bootloader */
        {
          Serial.println ("Bootloader");
        }
        Serial.println ();
      #endif 
      if (TEMP_INT == 1)                          /* Normal boot ? */
      {
        /* Answer request by opening main page */
        request->send(SPIFFS, "/Main.html", String(), false, SUB_WEB_PROCESSOR_MAIN); 
      }
      else                                        /* Bootloader */
      {
        BOOTLOADER_FLAG = true;                   /* Set bootloader flag true */
        /* Answer request by opening bootloader page */
        request->send(SPIFFS, "/Bootloader.html");
      }
    }
    else                                          /* Variable not found --> Reload page anyway */
    {  
      /* Answer request by opening main page */
      request->send(SPIFFS, "/Main.html", String(), false, SUB_WEB_PROCESSOR_MAIN); 
    }
  }); 

  /* Restore menu: Button for cancel pressed */
  server.on("/rcancel", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ("Restore menu: Cancel button pressed...");
      Serial.println ();
    #endif          
    /* Answer request with displaying of submain page 4 web page */
    request->send(SPIFFS, "/Submenu4.html", String(), false, SUB_WEB_PROCESSOR_SUBM4);
  });
  
  /* Restore menu: Button for save pressed */
  server.on("/rstore", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ("Restore menu: Store button pressed...");
      Serial.println ();
    #endif 
    SELECT = 143;                                 /* Set select to 143 --> Set default to storage positions */        
    /* Answer request with displaying of submain page 4 web page */
    request->send(SPIFFS, "/Submenu4.html", String(), false, SUB_WEB_PROCESSOR_SUBM4);
  });

  /*********************** Errorhandler on undefined routes ***************************/
  server.onNotFound(SUB_MESSAGE_ERROR);
}


/****************************************************************************************/
/* Subroutine to be called if message error is detected                                 */
/****************************************************************************************/
void SUB_MESSAGE_ERROR(AsyncWebServerRequest *request) 
{
    #ifdef _DEBUG_
      Serial.println ();               
      Serial.println("New client request...");
      Serial.println ("  Message error!!!");
      Serial.println ();
    #endif  
    request->send(404, "text/plain", "Message error");
}


/****************************************************************************************/
/* Subroutine to replace placeholder in website main with calculated values             */
/****************************************************************************************/
String SUB_WEB_PROCESSOR_MAIN (const String& REPLACE_TARGET)
{
  /* Definitions for dimming of color */
  const char *DIM_COLOR[] = { "100", "85", "70", "60", "50", "40", "30", "20", "10"};
  /* Definitions for dimming of program */
  const char *DIM_PROGRAM[] = { "100", "75", "50", "25"};
  /* Definitions for speed of program */
  const char *SPEED_PROGRAM[] = { "100", "90", "80", "70", "60", "50", "40", "30", "20", "10"};
  /* Definitions for fixed color */
  const char *NR_COLOR[] = { "Kaltweiss", "Warmweiss", "Gelb", "Orange", "Rot", "Dunkelviolett", "Violett", "Hellviolett", "Blau", "Hellblau", "Cyan", "Hellgrün", "Grün"};
  /* Definitions for storage positions */
  const char *NR_STORAGE[] = { "Position 1", "Position 2", "Position 3", "Position 4", "Position 5"};       
  /* Definitions for programs */
  const char *NR_PROGRAM[] = { "Farbwechsel 1", "Farbwechsel 2", "Farbwechsel 3", "Farbwechsel 4", "Farbwechsel 5", "Schritte", "Farbenspiel 1", "Farbenspiel 2", "Karminfeuer", "Gewitter"};   
  
  byte TEMP_BYTE;                                 /* Temporary byte for handling of strings */
    
  if (REPLACE_TARGET == "MAINTEXT1")              /* Actual time */
  {
    TEMP_BUFFER = TIME_STRING;                    /* Copy time in temp buffer */
    return (TEMP_BUFFER);                         /* Return message buffer */
  }
  if (REPLACE_TARGET == "MAINTEXT2")              /* Actual date */
  {
    TEMP_BUFFER = DATE_STRING;                    /* Copy date in temp buffer */
    return (TEMP_BUFFER);                         /* Return message buffer */
  } 
  if (REPLACE_TARGET == "MAINTEXT3")              /* Actual status on/off */
  {
    if (POWER_WEB_OFF == true)                    /* Power off status for web is true ? */
    {
      TEMP_BUFFER = "Aus";                        /* Set temp buffer to off */
    }
    else                                          /* Power off status is false? */
    {
      TEMP_BUFFER = "Ein";                        /* Set temp buffer to on */
    }    
    return (TEMP_BUFFER);                         /* Return message buffer */
  }   
  if (REPLACE_TARGET == "MAINTEXT4")              /* Actual program */
  {
    TEMP_BUFFER = "-";                            /* Predefine temp buffer */
    /* Color selected */
    if ((ACT_PROGRAM >= 1) && (ACT_PROGRAM <= 13))
    {
      TEMP_BUFFER = NR_COLOR[ACT_PROGRAM-1];
    }
    /* Storage selected */
    if ((ACT_PROGRAM >= 15) && (ACT_PROGRAM <= 19))
    {
      TEMP_BUFFER = NR_STORAGE[ACT_PROGRAM-15]; 
    }
    /* Program selected */
    if ((ACT_PROGRAM >= 20) && (ACT_PROGRAM <= 29))
    {
      TEMP_BUFFER = NR_PROGRAM[ACT_PROGRAM-20];  
    }
    return (TEMP_BUFFER);                         /* Return message buffer */
  }

  if (REPLACE_TARGET == "MAINTEXT5")              /* Actual color */
  {
    TEMP_BUFFER = "-";                            /* Predefine temp buffer */
    /* Color selected */
    if ((ACT_PROGRAM >= 1) && (ACT_PROGRAM <= 13))
    {
      TEMP_BUFFER = "Farbe";                      /* Set temp buffer to color */ 
    }
    /* Storage selected */
    if ((ACT_PROGRAM >= 15) && (ACT_PROGRAM <= 19))
    {
      TEMP_BUFFER = "Speicher";                   /* Set temp buffer to store */ 
    }
    /* Program selected */
    if ((ACT_PROGRAM >= 20) && (ACT_PROGRAM <= 29))
    {
      TEMP_BUFFER = "Programm";                   /* Set temp buffer to program */ 
    }
    return (TEMP_BUFFER);                         /* Return message buffer */      
  }
  
  if (REPLACE_TARGET == "MAINTEXT6")              /* Actual dimming color */
  {
    TEMP_BYTE = COLOR_DIM_STEP;                   /* Read actual color dimming value */
    TEMP_BUFFER = DIM_COLOR[TEMP_BYTE];
    return (TEMP_BUFFER);                         /* Return message buffer */ 
  }

  if (REPLACE_TARGET == "MAINTEXT7")              /* Actual program dimming value */
  {
    TEMP_BYTE  = PROG_DIM_STEP;                   /* Read actual program dimming value */
    TEMP_BUFFER = DIM_PROGRAM[TEMP_BYTE];
    return (TEMP_BUFFER);                         /* Return message buffer */ 
  }
  
  if (REPLACE_TARGET == "MAINTEXT8")              /* Actual program speed */
  {
    TEMP_BYTE  = PROG_SPEED;                      /* Read actual program dimming value */
    TEMP_BUFFER = SPEED_PROGRAM[TEMP_BYTE];
    return (TEMP_BUFFER);                         /* Return message buffer */ 
  }
    
  if (REPLACE_TARGET == "MAINTEXT9")              /* Actual mode */
  {
    TEMP_BYTE  = EEPROM.read (4);                 /* Read actual mode out of FLASH */
    if (TEMP_BYTE == 0)                           /* Mode is off ? */
    {                    
      TEMP_BUFFER = "Aus";
    }
    else                                          /* Mode is Automatic */
    {                    
      TEMP_BUFFER = "Zeitgesteuert";
    }    
    return (TEMP_BUFFER);                         /* Return message buffer */ 
  } 

  if (REPLACE_TARGET == "MAINTEXT10")             /* Mode slave 1 */
  {
    TEMP_BYTE  = EEPROM.read (5);                 /* Read actual mode slave 1 out of FLASH */
    if (TEMP_BYTE == 0)                           /* Slave 1 mode is off ? */
    {                    
      TEMP_BUFFER = "Aus";
    }
    else                                          /* Slave 1 mode is On */
    {                    
      TEMP_BUFFER = "Ein";
    }    
    return (TEMP_BUFFER);                         /* Return message buffer */ 
  }   
  
  if (REPLACE_TARGET == "MAINTEXT11")             /* Mode slave 2 */
  {
    TEMP_BYTE  = EEPROM.read (7);                 /* Read actual mode slave 2 out of FLASH */
    if (TEMP_BYTE == 0)                           /* Slave 2 mode is off ? */
    {                    
      TEMP_BUFFER = "Aus";
    }
    else                                          /* Slave 2 mode is On */
    {                    
      TEMP_BUFFER = "Ein";
    } 
    return (TEMP_BUFFER);                         /* Return message buffer */ 
  } 
  
  if (REPLACE_TARGET == "MAINTEXT12")             /* Mode slave 3 */
  {
    TEMP_BYTE  = EEPROM.read (9);                 /* Read actual mode slave 3 out of FLASH */
    if (TEMP_BYTE == 0)                           /* Slave 3 mode is off ? */
    {                    
      TEMP_BUFFER = "Aus";
    }
    else                                          /* Slave 3 mode is On */
    {                    
      TEMP_BUFFER = "Ein";
    }    
    return (TEMP_BUFFER);                         /* Return message buffer */ 
  }
  
  if (REPLACE_TARGET == "MAINTEXT13")             /* On time */
  {   
    TEMP_BUFFER = ON_TIME_HOUR;
    if (ON_TIME_MIN < 10)                         /* Display leading zero */
    {
      TEMP_BUFFER += ":0";
    }
    else
    {
      TEMP_BUFFER += ":";
    }
    TEMP_BUFFER += ON_TIME_MIN;
    return (TEMP_BUFFER);
  }
  
  if (REPLACE_TARGET == "MAINTEXT14")             /* Actual Ethernet IP address */
  {
    TEMP_BUFFER = ETH.localIP().toString();       /* Save actual LAN local IP address */
    return (TEMP_BUFFER);                         /* Return message buffer */
  }
  
  if (REPLACE_TARGET == "MAINTEXT15")             /* Actual Ethernet MAC-address */
  {
    TEMP_BUFFER = ETH.macAddress();               /* Save actual LAN MAC-address */
    return (TEMP_BUFFER);                         /* Return message buffer */
  }
  
  if (REPLACE_TARGET == "MAINTEXT16")             /* Actual Ethernet hostname */
  {
    TEMP_BUFFER = ETH.getHostname();              /* Save actual LAN host name */
    return (TEMP_BUFFER);                         /* Return message buffer */
  }  
  
  if (REPLACE_TARGET == "MAINTEXT17")             /* Actual LAN speed */
  {
    TEMP_BUFFER = ETH.linkSpeed();                /* Save actual LAN signal strength */
    TEMP_BUFFER += " Mbps";                       /* Add Mbps to string */
    return (TEMP_BUFFER);                         /* Return message buffer */
  } 
 
  if (REPLACE_TARGET == "MAINTEXT18")             /* Actual SW version */
  {
    TEMP_BUFFER = SW_VERSION;                     /* Save actual software version */
    return (TEMP_BUFFER);                         /* Return message buffer */
  }   
}


/****************************************************************************************/
/* Subroutine to replace placeholder in website submenu 1 with calculated values        */
/****************************************************************************************/
String SUB_WEB_PROCESSOR_SUBM1 (const String& REPLACE_TARGET)
{
  /* Definitions for NTP zone */
  const char *SUB1_NTPZONE[] = { "-5 Stunden", "-4 Stunden", "-3 Stunden", "-2 Stunden", "-1 Stunde", "0 Stunden", "+1 Stunde", "+2 Stunden", "+3 Stunden", "+4 Stunden", "+5 Stunden"};
  byte TEMP_BYTE;                                 /* Temporary byte for handling of strings */
  
  if (REPLACE_TARGET == "SUB1SELECT1")            /* Actual mode */
  {
    TEMP_BYTE  = EEPROM.read (4);                 /* Read actual mode out of FLASH */
    if (TEMP_BYTE == 0)                           /* Mode is off ? */
    {    
      RECEIVE_BUFFER = "<option value=\"0\" selected>Aus</option><br><option value=\"1\">Zeitgesteuert</option><br>";              
    }
    else                                          /* Mode is Automatic */
    {                    
      RECEIVE_BUFFER = "<option value=\"0\">Aus</option><br><option value=\"1\" selected>Zeitgesteuert</option><br>";  
    }    
    return (RECEIVE_BUFFER);                      /* Return message buffer */ 
  } 

  if (REPLACE_TARGET == "SUB1SELECT2")            /* Mode slave 1 */
  {
    TEMP_BYTE  = EEPROM.read (5);                 /* Read actual mode out of FLASH */
    if (TEMP_BYTE == 0)                           /* Slave 1 mode is off ? */
    {                    
      RECEIVE_BUFFER = "<option value=\"0\" selected>Aus</option><br><option value=\"1\">Ein</option><br>"; 
    }
    else                                          /* Slave 1 mode is On */
    {                    
      RECEIVE_BUFFER = "<option value=\"0\">Aus</option><br><option value=\"1\" selected>Ein</option><br>"; 
    }    
    return (RECEIVE_BUFFER);                      /* Return message buffer */ 
  }   
  
  if (REPLACE_TARGET == "SUB1SELECT3")            /* Mode slave 2 */
  {
    TEMP_BYTE  = EEPROM.read (7);                 /* Read actual mode out of FLASH */
    if (TEMP_BYTE == 0)                           /* Slave 2 mode is off ? */
    {                    
      RECEIVE_BUFFER = "<option value=\"0\" selected>Aus</option><br><option value=\"1\">Ein</option><br>";
    }
    else                                          /* Slave 2 mode is On */
    {                    
      RECEIVE_BUFFER = "<option value=\"0\">Aus</option><br><option value=\"1\" selected>Ein</option><br>";
    } 
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  } 
  
  if (REPLACE_TARGET == "SUB1SELECT4")            /* Mode slave 3 */
  {
    TEMP_BYTE  = EEPROM.read (9);                 /* Read actual mode out of FLASH */
    if (TEMP_BYTE == 0)                           /* Slave 3 mode is off ? */
    {                    
      RECEIVE_BUFFER = "<option value=\"0\" selected>Aus</option><br><option value=\"1\">Ein</option><br>";
    }
    else                                          /* Slave 3 mode is On */
    {                    
      RECEIVE_BUFFER = "<option value=\"0\">Aus</option><br><option value=\"1\" selected>Ein</option><br>";
    }    
    return (RECEIVE_BUFFER);                      /* Return message buffer */ 
  }

  if (REPLACE_TARGET == "SUB1VALUE1")             /* NTP time hour value */
  {
    RECEIVE_BUFFER = NTP_UPDATE_HOUR;             /* Convert NTP update hour to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }   

  if (REPLACE_TARGET == "SUB1VALUE2")             /* NTP time minute value */
  {
    RECEIVE_BUFFER = NTP_UPDATE_MIN;              /* Convert NTP update minute to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }   

  if (REPLACE_TARGET == "SUB1SELECT5")            /* NTP time zone */
  {
    RECEIVE_BUFFER = "";                          /* ´Delete replacement string */
    /* Generate replacement string in loop for all options of display timeout */
    for(LOOP_COUNTER=0; LOOP_COUNTER <= 10; LOOP_COUNTER++)
    {
      RECEIVE_BUFFER += "<option value=\"";
      RECEIVE_BUFFER += LOOP_COUNTER;
      RECEIVE_BUFFER += "\"";
      if (NTP_TIME_ZONE == LOOP_COUNTER)          /* Default selection ? */
      {
        RECEIVE_BUFFER += " selected>";
      }
      else                                        /* No default selection ? */
      {
        RECEIVE_BUFFER += ">";
      } 
      RECEIVE_BUFFER += SUB1_NTPZONE[LOOP_COUNTER];
      RECEIVE_BUFFER += "</option><br>";
    }
    return (RECEIVE_BUFFER);                      /* Return message buffer */ 
  }
  
  if (REPLACE_TARGET == "SUB1SELECT6")            /* NTP automatic summertime switching */
  { 
    if (NTP_SUMMERTIME == false)                  /* No automatic summertime switching ? */
    {
      RECEIVE_BUFFER = "<option value=\"1\" selected>Nein</option><br><option value=\"2\">Ja</option><br>";
    }
    else                                          /* Automatic summertime switching ? */
    {
      RECEIVE_BUFFER = "<option value=\"1\">Nein</option><br><option value=\"2\" selected>Ja</option><br>";
    }    
    return (RECEIVE_BUFFER);                      /* Return message buffer */ 
  }
}

/****************************************************************************************/
/* Subroutine to replace placeholder in website submenu 2 with calculated values        */
/****************************************************************************************/
String SUB_WEB_PROCESSOR_SUBM2 (const String& REPLACE_TARGET)
{
  /* Definitions for dimming of color */
  const char *FIX_DIM[] = { "100", "85", "70", "60", "50", "40", "30", "20", "10"};
  /* Definitions for fixed color */
  const char *FIX_COLOR[] = { "---", "Kaltweiss", "Warmweiss", "Gelb", "Orange", "Rot", "Dunkelviolett", "Violett", "Hellviolett", "Blau", "Hellblau", "Cyan", "Hellgrün", "Grün"};
  byte TEMP_BYTE;                                 /* Temporary byte to determine right position */

  if (REPLACE_TARGET == "SUB2SELECT1")            /* Actual color */
  {
    /* Color selected */
    if ((ACT_PROGRAM >= 1) && (ACT_PROGRAM <= 13))
    {
      RECEIVE_BUFFER = "";                        /* ´Delete replacement string */
      /* Generate replacement string in loop for all options of watering mode */
      for(LOOP_COUNTER=1; LOOP_COUNTER <= 13; LOOP_COUNTER++)
      {
        RECEIVE_BUFFER += "<option value=\"";
        RECEIVE_BUFFER += LOOP_COUNTER;
        RECEIVE_BUFFER += "\"";
        if (ACT_PROGRAM == LOOP_COUNTER)          /* Default selection ? */
        {
          RECEIVE_BUFFER += " selected>";
        }
        else                                      /* No default selection ? */
        {
          RECEIVE_BUFFER += ">";
        }
        RECEIVE_BUFFER += FIX_COLOR[LOOP_COUNTER];
        RECEIVE_BUFFER += "</option><br>";
      }
    }
    else                                          /* Program or storage selected ? */
    {
      RECEIVE_BUFFER = "";                        /* ´Delete replacement string */
      /* Generate replacement string in loop for all options of watering mode */
      for(LOOP_COUNTER=0; LOOP_COUNTER <= 13; LOOP_COUNTER++)
      {
        RECEIVE_BUFFER += "<option value=\"";
        RECEIVE_BUFFER += LOOP_COUNTER;
        RECEIVE_BUFFER += "\"";
        if (LOOP_COUNTER == 0)                    /* Default selection ? */
        {
          RECEIVE_BUFFER += " selected>";
        }
        else                                      /* No default selection ? */
        {
          RECEIVE_BUFFER += ">";
        }
        RECEIVE_BUFFER += FIX_COLOR[LOOP_COUNTER];
        RECEIVE_BUFFER += "</option><br>";
      }
    }
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  
  if (REPLACE_TARGET == "SUB2SELECT2")            /* Actual dim factor for color */
  {
    RECEIVE_BUFFER = "";                          /* ´Delete replacement string */
    /* Generate replacement string in loop for all options of dimming for color */
    for(LOOP_COUNTER=0; LOOP_COUNTER <= 8; LOOP_COUNTER++)
    {
      RECEIVE_BUFFER += "<option value=\"";
      RECEIVE_BUFFER += LOOP_COUNTER;
      RECEIVE_BUFFER += "\"";
      if (COLOR_DIM_STEP == LOOP_COUNTER)         /* Default selection ? */
      {
        RECEIVE_BUFFER += " selected>";
      }
      else                                        /* No default selection ? */
      {
        RECEIVE_BUFFER += ">";
      }
      RECEIVE_BUFFER += FIX_DIM[LOOP_COUNTER];
      RECEIVE_BUFFER += "</option><br>";
    }
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
}

/****************************************************************************************/
/* Subroutine to replace placeholder in website submenu 3 with calculated values        */
/****************************************************************************************/
String SUB_WEB_PROCESSOR_SUBM3 (const String& REPLACE_TARGET)
{
  /* Definitions for programs */
  const char *PR_MODE[] = { "---", "Farbwechsel 1", "Farbwechsel 2", "Farbwechsel 3", "Farbwechsel 4", "Farbwechsel 5", "Schritte", "Farbenspiel 1", "Farbenspiel 2", "Karminfeuer", "Gewitter"};   
  /* Definitions for dimming of color */
  const char *PR_DIM[] = { "100", "75", "50", "25"};
  /* Definitions for program speed */
  const char *PR_SPEED[] = { "100", "90", "80", "70", "60", "50", "40", "30", "20", "10"};
  
  byte TEMP_BYTE;                                 /* Temporary byte to determine right position */

  if (REPLACE_TARGET == "SUB3SELECT1")            /* Actual color */
  {
    /* Program selected */
    if ((ACT_PROGRAM >= 20) && (ACT_PROGRAM <= 29))   
    {
      TEMP_BYTE = ACT_PROGRAM - 19;               /* Calculate right offset */
      RECEIVE_BUFFER = "";                        /* ´Delete replacement string */
      /* Generate replacement string in loop for all options of watering mode */
      for(LOOP_COUNTER=1; LOOP_COUNTER <= 10; LOOP_COUNTER++)
      {
        RECEIVE_BUFFER += "<option value=\"";
        RECEIVE_BUFFER += LOOP_COUNTER;
        RECEIVE_BUFFER += "\"";
        if (LOOP_COUNTER == TEMP_BYTE)            /* Default selection ? */
        {
          RECEIVE_BUFFER += " selected>";
        }
        else                                      /* No default selection ? */
        {
          RECEIVE_BUFFER += ">";
        }
        RECEIVE_BUFFER += PR_MODE[LOOP_COUNTER];
        RECEIVE_BUFFER += "</option><br>";
      }
    }
    else                                          /* Single color or storage selected ? */
    {
      RECEIVE_BUFFER = "";                        /* ´Delete replacement string */
      /* Generate replacement string in loop for all options of watering mode */
      for(LOOP_COUNTER=0; LOOP_COUNTER <= 10; LOOP_COUNTER++)
      {
        RECEIVE_BUFFER += "<option value=\"";
        RECEIVE_BUFFER += LOOP_COUNTER;
        RECEIVE_BUFFER += "\"";
        if (LOOP_COUNTER == 0)                    /* Default selection ? */
        {
          RECEIVE_BUFFER += " selected>";
        }
        else                                      /* No default selection ? */
        {
          RECEIVE_BUFFER += ">";
        }
        RECEIVE_BUFFER += PR_MODE[LOOP_COUNTER];
        RECEIVE_BUFFER += "</option><br>";
      }
    }
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  
  if (REPLACE_TARGET == "SUB3SELECT2")            /* Actual dimming factor for program */
  {
    RECEIVE_BUFFER = "";                          /* ´Delete replacement string */
    /* Generate replacement string in loop for all options of program dimming */
    for(LOOP_COUNTER=0; LOOP_COUNTER <= 3; LOOP_COUNTER++)
    {
      RECEIVE_BUFFER += "<option value=\"";
      RECEIVE_BUFFER += LOOP_COUNTER;
      RECEIVE_BUFFER += "\"";
      if (LOOP_COUNTER == PROG_DIM_STEP)          /* Default selection ? */
      {
        RECEIVE_BUFFER += " selected>";
      }
      else                                        /* No default selection ? */
      {
        RECEIVE_BUFFER += ">";
      }
      RECEIVE_BUFFER += PR_DIM[LOOP_COUNTER];
      RECEIVE_BUFFER += "</option><br>";
    }
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }

  if (REPLACE_TARGET == "SUB3SELECT3")            /* Actual speed factor for program */
  {
    RECEIVE_BUFFER = "";                          /* ´Delete replacement string */
    /* Generate replacement string in loop for all options of program speed */
    for(LOOP_COUNTER=0; LOOP_COUNTER <= 9; LOOP_COUNTER++)
    {
      RECEIVE_BUFFER += "<option value=\"";
      RECEIVE_BUFFER += LOOP_COUNTER;
      RECEIVE_BUFFER += "\"";
      if (LOOP_COUNTER == PROG_SPEED)             /* Default selection ? */
      {
        RECEIVE_BUFFER += " selected>";
      }
      else                                        /* No default selection ? */
      {
        RECEIVE_BUFFER += ">";
      }
      RECEIVE_BUFFER += PR_SPEED[LOOP_COUNTER];
      RECEIVE_BUFFER += "</option><br>";
    }
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
}

/****************************************************************************************/
/* Subroutine to replace placeholder in website submenu 4 with calculated values        */
/****************************************************************************************/
String SUB_WEB_PROCESSOR_SUBM4 (const String& REPLACE_TARGET)
{
  /* Definitions for stores 1-5 */
  const char *STORE_MOD[] = { "---", "Speicher 1", "Speicher 2", "Speicher 3", "Speicher 4", "Speicher 5"};
  /* Definitions for dimming of color */
  const char *STORE_DIM[] = { "100", "85", "70", "60", "50", "40", "30", "20", "10"};

  byte TEMP_BYTE;                                 /* Temporary byte to determine right position */

  if (REPLACE_TARGET == "SUB4SELECT1")            /* Actual store */
  {
    /* Store selected */
    if ((ACT_PROGRAM >= 15) && (ACT_PROGRAM <= 19))   
    {
      TEMP_BYTE = ACT_PROGRAM - 14;               /* Calculate right offset */
      RECEIVE_BUFFER = "";                        /* ´Delete replacement string */
      /* Generate replacement string in loop for all options of stores */
      for(LOOP_COUNTER=1; LOOP_COUNTER <= 5; LOOP_COUNTER++)
      {
        RECEIVE_BUFFER += "<option value=\"";
        RECEIVE_BUFFER += LOOP_COUNTER;
        RECEIVE_BUFFER += "\"";
        if (LOOP_COUNTER == TEMP_BYTE)            /* Default selection ? */
        {
          RECEIVE_BUFFER += " selected>";
        }
        else                                      /* No default selection ? */
        {
          RECEIVE_BUFFER += ">";
        }
        RECEIVE_BUFFER += STORE_MOD[LOOP_COUNTER];
        RECEIVE_BUFFER += "</option><br>";
      }
    }
    else                                          /* Single color or storage selected ? */
    {
      RECEIVE_BUFFER = "";                        /* ´Delete replacement string */
      /* Generate replacement string in loop for all options of stores */
      for(LOOP_COUNTER=0; LOOP_COUNTER <= 5; LOOP_COUNTER++)
      {
        RECEIVE_BUFFER += "<option value=\"";
        RECEIVE_BUFFER += LOOP_COUNTER;
        RECEIVE_BUFFER += "\"";
        if (LOOP_COUNTER == 0)                    /* Default selection ? */
        {
          RECEIVE_BUFFER += " selected>";
        }
        else                                      /* No default selection ? */
        {
          RECEIVE_BUFFER += ">";
        }
        RECEIVE_BUFFER += STORE_MOD[LOOP_COUNTER];
        RECEIVE_BUFFER += "</option><br>";
      }
    }
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }

  if (REPLACE_TARGET == "SUB4VALUE1")             /* Red color value */
  {  
    if (SUB_MENU_FLAG == false)                   /* Display zero values */
    {
      RECEIVE_BUFFER = "0";                       /* Set receive buffer with 0 */
    } 
    else                                          /* Load actual valus from restore */
    {
      RECEIVE_BUFFER = STORE_RED_COLOR;           /* Copy actual storage red value into receive buffer */ 
    }
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }

  if (REPLACE_TARGET == "SUB4VALUE2")             /* Green color value */
  {  
    if (SUB_MENU_FLAG == false)                   /* Display zero values */
    {    
      RECEIVE_BUFFER = "0";                       /* Set receive buffer with 0 */
    }
    else                                          /* Load actual valus from restore */
    {
      RECEIVE_BUFFER = STORE_GREEN_COLOR;         /* Copy actual storage green value into receive buffer */ 
    }   
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  } 
  
  if (REPLACE_TARGET == "SUB4VALUE3")             /* Blue color value */
  {  
    if (SUB_MENU_FLAG == false)                   /* Display zero values */
    {    
      RECEIVE_BUFFER = "0";                       /* Set receive buffer with 0 */
    }
    else                                          /* Load actual valus from restore */
    {
      RECEIVE_BUFFER = STORE_BLUE_COLOR;          /* Copy actual storage blue value into receive buffer */ 
    }   
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  } 
      

  if (REPLACE_TARGET == "SUB4SELECT2")            /* Actual dim factor for storage */
  {
    RECEIVE_BUFFER = "";                          /* ´Delete replacement string */
    if (SUB_MENU_FLAG == false)                   /* Display default 0 values */
    {    
      for(LOOP_COUNTER=0; LOOP_COUNTER <= 8; LOOP_COUNTER++)
      {
        RECEIVE_BUFFER += "<option value=\"";
        RECEIVE_BUFFER += LOOP_COUNTER;
        RECEIVE_BUFFER += "\"";
        if (LOOP_COUNTER == 0)                    /* Default selection ? */
        {
          RECEIVE_BUFFER += " selected>";
        }
        else                                      /* No default selection ? */
        {
          RECEIVE_BUFFER += ">";
        }
        RECEIVE_BUFFER += STORE_DIM[LOOP_COUNTER];
        RECEIVE_BUFFER += "</option><br>";
      }
    }
    else                                          /* Load actual valus from restore */
    {  
      /* Generate replacement string in loop for all options of dimming for color */
      for(LOOP_COUNTER=0; LOOP_COUNTER <= 8; LOOP_COUNTER++)
      {
        RECEIVE_BUFFER += "<option value=\"";
        RECEIVE_BUFFER += LOOP_COUNTER;
        RECEIVE_BUFFER += "\"";
        if (LOOP_COUNTER == STORE_DIM_VALUE)      /* Default selection ? */
        {
          RECEIVE_BUFFER += " selected>";
        }
        else                                      /* No default selection ? */
        {
          RECEIVE_BUFFER += ">";
        }
        RECEIVE_BUFFER += STORE_DIM[LOOP_COUNTER];
        RECEIVE_BUFFER += "</option><br>";
      }
    }
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
}


/****************************************************************************************/
/* Subroutine to replace placeholder in website submenu 6 with calculated values        */
/****************************************************************************************/
String SUB_WEB_PROCESSOR_SUBM5 (const String& REPLACE_TARGET)
{
  if (REPLACE_TARGET == "SUB5VALUE1")             /* Hostname */
  {
    RECEIVE_BUFFER = HOSTNAME;                    /* Convert Hostname to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE2")             /* Fixed IP 4th digit */
  {
    RECEIVE_BUFFER = FIXED_IP[0];                 /* Convert Fixed IP 4th digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE3")             /* Fixed IP 3rd digit */
  {
    RECEIVE_BUFFER = FIXED_IP[1];                 /* Convert Fixed IP 3rd digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE4")             /* Fixed IP 2nd digit */
  {
    RECEIVE_BUFFER = FIXED_IP[2];                 /* Convert Fixed IP 2nd digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE5")             /* Fixed IP 1st digit */
  {
    RECEIVE_BUFFER = FIXED_IP[3];                 /* Convert Fixed IP 1st digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }    
  if (REPLACE_TARGET == "SUB5VALUE6")             /* Subnet mask 4th digit */
  {
    RECEIVE_BUFFER = SUBNET[0];                   /* Convert Subnet mask 4th digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE7")             /* Subnet mask 3rd digit */
  {
    RECEIVE_BUFFER = SUBNET[1];                   /* Convert Subnet mask 3rd digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE8")             /* Subnet mask 2nd digit */
  {
    RECEIVE_BUFFER = SUBNET[2];                   /* Convert Subnet mask 2nd digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE9")             /* Subnet mask 1st digit */
  {
    RECEIVE_BUFFER = SUBNET[3];                   /* Convert Subnet mask 1st digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE10")            /* Gateway IP 4th digit */
  {
    RECEIVE_BUFFER = GATEWAY[0];                  /* Convert Gateway IP 4th digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE11")            /* Gateway IP 3rd digit */
  {
    RECEIVE_BUFFER = GATEWAY[1];                  /* Convert Gateway IP 3rd digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE12")            /* Gateway IP 2nd digit */
  {
    RECEIVE_BUFFER = GATEWAY[2];                  /* Convert Gateway IP 2nd digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE13")            /* Gateway IP 1st digit */
  {
    RECEIVE_BUFFER = GATEWAY[3];                  /* Convert Gateway IP 1st digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE14")            /* DNA 1 IP 4th digit */
  {
    RECEIVE_BUFFER = DNS_1[0];                    /* Convert DNS 1 IP 4th digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE15")            /* DNA 1 IP 3rd digit */
  {
    RECEIVE_BUFFER = DNS_1[1];                    /* Convert DNS 1 IP 3rd digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE16")            /* DNA 1 IP 2nd digit */
  {
    RECEIVE_BUFFER = DNS_1[2];                    /* Convert DNS 1 IP 2nd digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE17")            /* DNA 1 IP 1st digit */
  {
    RECEIVE_BUFFER = DNS_1[3];                    /* Convert DNS 1 IP 1st digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE18")            /* DNA 2 IP 4th digit */
  {
    RECEIVE_BUFFER = DNS_2[0];                    /* Convert DNS 2 IP 4th digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE19")            /* DNA 2 IP 3rd digit */
  {
    RECEIVE_BUFFER = DNS_2[1];                    /* Convert DNS 2 IP 3rd digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE20")            /* DNA 2 IP 2nd digit */
  {
    RECEIVE_BUFFER = DNS_2[2];                    /* Convert DNS 2 IP 2nd digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE21")            /* DNA 2 IP 1st digit */
  {
    RECEIVE_BUFFER = DNS_2[3];                    /* Convert DNS 2 IP 1st digit to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE22")            /* NTP 1 server name */
  {
    RECEIVE_BUFFER = NTP_SERVER_1;                /* Convert NTP server 1 name to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE23")            /* NTP 2 server name */
  {
    RECEIVE_BUFFER = NTP_SERVER_2;                /* Convert NTP server 2 name to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  if (REPLACE_TARGET == "SUB5VALUE24")            /* NTP 2 server name */
  {
    RECEIVE_BUFFER = NTP_SERVER_3;                /* Convert NTP server 3 name to string */
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }        
}

/****************************************************************************************/
/* Subroutine to replace placeholder in website submenu 6 with calculated values        */
/* For options see example for mode string to be returned                               */
/* <option value="1" selected>Normal</option><br><option value="2">Bootloader           */
/* </option><br>                                                                        */
/* Note that " must be \" in the string!                                                */
/****************************************************************************************/
String SUB_WEB_PROCESSOR_SUBM6 (const String& REPLACE_TARGET)
{
  if (REPLACE_TARGET == "SUB6SELECT1")            /* Boot mode */
  {
    /* Replacement string */
    RECEIVE_BUFFER = "<option value=\"1\" selected>Normal</option><br><option value=\"2\">Bootloader</option><br>";
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
}
